@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.14 10/.0/.1 .0:.0:.5 abe Exp $
+ * $Id: dlsof.h,v 1.36 2004/12/30 18:41:31 abe Exp $
  */
 
 
@@ -46,15 +46,46 @@
 #include <string.h>
 #include <unistd.h>
 
-# if	FREEBSDV>=400
+# if	FREEBSDV>=4000
+#  if	FREEBSDV>=5000
+#    if	defined(__alpha__)
+/*
+ * For Alpha, #include <machine/pcpu.h> before #define'ing _KERNEL.  Then
+ * #define PCPU_MD_FIELDS independently.  This hack avoids a compiler
+ * complaint about register use.
+ */
+
+#include <machine/pcpu.h>       
+#define PCPU_MD_FIELDS                                                  \
+	struct alpha_pcb pc_idlepcb;            /* pcb for idling */    \
+	u_int64_t       pc_idlepcbphys;         /* pa of pc_idlepcb */  \
+	u_int64_t       pc_pending_ipis;        /* pending IPI's */     \
+	u_int32_t       pc_next_asn;            /* next ASN to alloc */ \
+	u_int32_t       pc_current_asngen       /* ASN rollover check */
+#    endif	/* defined(__alpha__) */
+#define	_KERNEL	1
+#  endif	/* FREEBSDV>=5000 */
 #include <sys/conf.h>
-# endif	/* FREEBSDV>=400 */
+#  if	FREEBSDV>=5000
+#undef	_KERNEL
+#  endif	/* FREEBSDV>=5000 */
+# endif	/* FREEBSDV>=4000 */
 
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
 #define	NFS
 #define m_stat	mnt_stat
+
+# if	FREEBSDV>=3020
+#define	_KERNEL
+# endif	/* FREEBSDV>=3020 */
+
 #include <sys/mount.h>
+
+# if	FREEBSDV>=3020
+#undef	_KERNEL
+# endif	/* FREEBSDV>=3020 */
+
 #include <rpc/types.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
@@ -62,9 +93,9 @@
 #include <sys/un.h>
 #include <sys/unpcb.h>
 
-# if	FREEBSDV>=300
+# if	FREEBSDV>=3000
 #undef	INADDR_LOOPBACK
-# endif	/* FREEBSDV>=300 */
+# endif	/* FREEBSDV>=3000 */
 
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -79,7 +110,15 @@
 #include <netinet/tcp_var.h>
 #include <sys/ucred.h>
 #include <sys/uio.h>
+
+# if	defined(HAS_KVM_VNODE)
+#define	_KVM_VNODE
+# endif	/* defined(HAS_KVM_VNODE) */
 #include <sys/vnode.h>
+# if	defined(HAS_KVM_VNODE)
+#undef	_KVM_VNODE
+# endif	/* defined(HAS_KVM_VNODE) */
+
 #include <net/raw_cb.h>
 #include <sys/domain.h>
 #define	pmap	RPC_pmap
@@ -87,86 +126,139 @@
 #include <rpc/pmap_prot.h>
 #undef	pmap
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 #include <ufs/quota.h>
 #include <ufs/inode.h>
 #include <ufs/ufsmount.h>
 #include <ufs/mfsnode.h>
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 #include <paths.h>
 #include <ufs/ufs/quota.h>
 
-#  if	FREEBSDV>=400 && defined(__alpha__)
+#  if	FREEBSDV>=4000 && FREEBSDV<5000
+#   if	defined(__alpha__) || defined(__sparc64__)
 #define	dev_t	void *
-#  endif	/* FREEBSDV>=400 && defined(__alpha__) */
+#   endif	/* defined(__alpha__) || defined(__sparc64__) */
+#  endif /* FREEBSDV>=4000 && FREEBSDV<5000 */
 
 #include <ufs/ufs/inode.h>
 
-#  if	FREEBSDV>=400 && defined(__alpha__)
+#  if	FREEBSDV>=5010
+#undef	i_devvp
+#  endif	/* FREEBSDV>=5010 */
+
+#  if	FREEBSDV>=4000 && FREEBSDV<5000
+#   if	defined(__alpha__) || defined(__sparc64__)
 #undef	dev_t
-#  endif	/* FREEBSDV>=400 && defined(__alpha__) */
+#   endif	/* defined(__alpha__) || defined(__sparc64__) */
+#  endif /* FREEBSDV>=4000 && FREEBSDV<5000 */
 
-#  if   FREEBSDV<220
+#  if   FREEBSDV<2020
 #include <ufs/mfs/mfsnode.h>
-#  endif        /* FREEBSDV<220 */
+#  endif        /* FREEBSDV<2020 */
 
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
+# if	FREEBSDV<5000
 #include <nfs/nfsv2.h>
+# else	/* FREEBSDV>=5000 */
+#include <nfs/nfsproto.h>
+# endif	/* FREEBSDV<5000 */
 
-# if	defined(HASRPCV2H) || FREEBSDV>=400
+# if	defined(HASRPCV2H) || FREEBSDV>=4000
 #include <nfs/rpcv2.h>
-# endif	/* defined(HASRPCV2H) || FREEBSDV>=400 */
+# endif	/* defined(HASRPCV2H) || FREEBSDV>=4000 */
 
+# if	FREEBSDV>=5000
+#include <nfsclient/nfs.h>
+#include <nfsclient/nfsnode.h>
+# else	/* FREEBSDV<5000 */
 #include <nfs/nfs.h>
 #include <nfs/nfsnode.h>
+# endif	/* FREEBSDV>=5000 */
+
 #include <sys/proc.h>
 #include <kvm.h>
 #undef	TRUE
 #undef	FALSE
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 #include <sys/kinfo.h>
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 #include <sys/sysctl.h>
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
 # if	defined(HASFDESCFS)
 #define	_KERNEL
 #define	KERNEL
+#  if	FREEBSDV>=5000
+#include <fs/fdescfs/fdesc.h>
+#  else	/* FREEBSDV<5000 */
 #include <miscfs/fdesc/fdesc.h>
+#  endif	/* FREEBSDV>=5000 */
 #undef	_KERNEL
 #undef	KERNEL
 # endif	/* defined(HASFDESCFS) */
 
+# if	defined(HASNULLFS)
+#define	_KERNEL
+#define	KERNEL
+struct vop_generic_args;
+#  if	FREEBSDV>=5000
+#include <fs/nullfs/null.h>
+#  else	/* FREEBSDV<5000 */
+#include <miscfs/nullfs/null.h>
+#  endif	/* FREEBSDV>=5000 */
+#undef	_KERNEL
+#undef	KERNEL
+# endif	/* defined(HASNULLFS) */
+
 # if	defined(HASPROCFS)
-#  if	FREEBSDV<200
+#  if	FREEBSDV<2000
 #include <procfs/pfsnode.h>
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
+#  if	FREEBSDV<5000
 #include <miscfs/procfs/procfs.h>
+#  endif	/* FREEBSDV<5000 */
 #include <machine/reg.h>
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
 #define	PNSIZ		5
 # endif	/* defined(HASPROCFS) */
 
-# if	FREEBSDV<200
+# if	defined(HASPSEUDOFS)
+#include <fs/pseudofs/pseudofs.h>
+# endif	/* defined(HASPSEUDOFS) */
+
+
+# if	FREEBSDV<2000
 #define	P_COMM		p_comm
 #define	P_FD		p_fd
 #define	P_PID		p_pid
-#define	P_PGRP		p_pgrp
+#define	P_PGID		p_pgrp
 #define	P_STAT		p_stat
 #define	P_VMSPACE	p_vmspace
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
+#  if	FREEBSDV<5000
 #define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
 #define	P_FD		kp_proc.p_fd
 #define	P_PID		kp_proc.p_pid
-#define	P_PGRP		kp_eproc.e_pgid
+#define	P_PGID		kp_eproc.e_pgid
 #define	P_PPID		kp_eproc.e_ppid
 #define	P_STAT		kp_proc.p_stat
 #define	P_VMSPACE	kp_proc.p_vmspace
-# endif	/* FREEBSDV<200 */
+#  else	/* FREEBSDV>=5000 */
+#define	P_ADDR		ki_paddr
+#define	P_COMM		ki_comm
+#define	P_FD		ki_fd
+#define	P_PID		ki_pid
+#define	P_PGID		ki_pgid
+#define	P_PPID		ki_ppid
+#define	P_STAT		ki_stat
+#define	P_VMSPACE	ki_vmspace
+#  endif	/* FREEBSDV<5000 */
+# endif	/* FREEBSDV<2000 */
 
 #define	_KERNEL
 #define	KERNEL
@@ -175,22 +267,47 @@
 #undef	_KERNEL
 #undef	KERNEL
 
-# if	FREEBSDV<200
+# if	defined(DTYPE_KQUEUE)
+#define	HASKQUEUE				/* has the kqueue file type */
+#   if	FREEBSDV>=4090
+#define	_KERNEL
+#   endif	/* FREEBSDV>=4090 */
+#include <sys/eventvar.h>
+#   if	FREEBSDV>=4090
+#undef	_KERNEL
+#   endif	/* FREEBSDV>=4090 */
+# endif	/* defined(DTYPE_KQUEUE) */
+
+# if	FREEBSDV<2000
 #include <ufs/lockf.h>
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
+#  if	FREEBSDV>=5000
+#undef	MALLOC_DECLARE
+#define	MALLOC_DECLARE(type)	extern struct malloc_type type[1]
+					/* to pacify <sys/lockf.h> */
+#define	_KERNEL
+#include <fs/devfs/devfs.h>
+#undef	_KERNEL
+#  endif	/* FREEBSDV>=5000 */
 #include <sys/lockf.h>
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
 #include <vm/vm.h>
 
-#  if   FREEBSDV>=220
+#  if   FREEBSDV>=2020
+#   if	FREEBSDV>=4090
+#define	_KERNEL
+#   endif	/* FREEBSDV>=4090 */
 #include <sys/pipe.h>
+#   if	FREEBSDV>=4090
+#undef	_KERNEL
+#   endif	/* FREEBSDV>=4090 */
 #   if	defined(HASVMLOCKH)
 #include <vm/lock.h>
 #   endif	/* defined(HASVMLOCKH) */
 #include <vm/pmap.h>
-#  endif        /* FREEBSDV>=220 */
+#  endif        /* FREEBSDV>=2020 */
 
 #include <vm/vm_map.h>
 
@@ -206,22 +323,30 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #include <vm/vm_object.h>
 #include <vm/vm_pager.h>
 
-#  if   FREEBSDV>=220
+#  if   FREEBSDV>=2020
 #undef	B_NEEDCOMMIT
+
+#   if	FREEBSDV>=5000
+#include <sys/bio.h>
+#   endif	/* FREEBSDV>=5000 */
+
 #include <sys/buf.h>
 #include <sys/user.h>
+
+#   if	FREEBSDV<5000
 #include <ufs/mfs/mfsnode.h>
-#  endif        /* FREEBSDV>=220 */
+#   endif	/* FREEBSDV<5000 */
+#  endif        /* FREEBSDV>=2020 */
 
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 typedef	off_t		KA_T;
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 typedef	u_long		KA_T;
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
@@ -238,6 +363,10 @@ typedef	u_long		KA_T;
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
 #define SWAP		"/dev/drum"
+#define	SZOFFTYPE	long long	/* size and offset internal storage
+					 * type */
+#define	SZOFFPSPEC	"ll"		/* SZOFFTYPE print specification
+					 * modifier */
 
 
 /*
@@ -246,9 +375,9 @@ typedef	u_long		KA_T;
 
 struct file * Cfp;
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 extern kvm_t *Kd;
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 # if	defined(P_ADDR)
 extern KA_T Kpa;
@@ -290,15 +419,16 @@ struct mounts {
 
 extern int Np;				/* number of kernel processes */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 extern struct kinfo_proc *P;		/* local process table copy */
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 struct sfile {
 	char *aname;			/* argument file name */
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
+	dev_t rdev;			/* raw device */
 	u_short mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
@@ -308,6 +438,16 @@ struct sfile {
 
 };
 
+# if	FREEBSDV==4100
+#define	XDR_VOID	(xdrproc_t)xdr_void
+#define	XDR_PMAPLIST	(xdrproc_t)xdr_pmaplist
+# endif	/* FREEBSDV==4100 */
+
+# if	FREEBSDV>=5000
+#define	XDR_VOID	(const xdrproc_t)xdr_void
+#define	XDR_PMAPLIST	(const xdrproc_t)xdr_pmaplist
+# endif	/* FREEBSDV>=5000 */
+
 
 /*
  * Definitions for rdev.c
@@ -318,14 +458,14 @@ struct sfile {
 
 
 /*
- * Definitions for rnam.c
+ * Definitions for rnam.c and rnmh.c
  */
 
 # if     defined(HASNCACHE)
 #include <sys/uio.h>
-#  if	FREEBSDV<400 || (FREEBSDV>=400 && defined(HASNAMECACHE))
+#  if	FREEBSDV<4000 || (FREEBSDV>=4000 && defined(HASNAMECACHE))
 #include <sys/namei.h>
-#  else	/* FREEBSDV>=400 && !defined(HASNAMECACHE) */
+#  else	/* FREEBSDV>=4000 && !defined(HASNAMECACHE) */
 /*
  * The namecache struct definition should come from a header file that
  * can be #include'd, but it has been moved to a kernel source file in
@@ -342,24 +482,34 @@ struct	namecache {
 	struct	vnode *nc_vp;		/* vnode the name refers to */
 	u_char	nc_flag;		/* flag bits */
 	u_char	nc_nlen;		/* length of name */
-	char	nc_name[0];		/* segment name */
+	char	nc_name[16];		/* segment name -- Strictly composed,
+					 * the size of nc_name[] should be zero
+					 * and rnmh.c in lsof/lib should read
+					 * the name with a separate call to
+					 * kvm_read().  Since that causes extra
+					 * (and slow) calls to kvm_read(), the
+					 * size is set here to an experimentally
+					 * derived guess.  The same experiment
+					 * didn't reveal any extra kvm_read()
+					 * suggesting the guess is a safe one.
+					 * (VAA, 10 Apr 2002) */
 };
-#  endif	/* FREEBSDV<400 || (FREEBSDV>=400 && defined(HASNAMECACHE)) */
+#  endif	/* FREEBSDV<4000 || (FREEBSDV>=4000 && defined(HASNAMECACHE)) */
 
 #define	NCACHE		namecache	/* kernel's structure name */
 #define	NCACHE_NM	nc_name		/* name in NCACHE */
 #define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
 
-#  if	FREEBSDV<205
+#  if	FREEBSDV<2005
 #define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
-#  else	/* FREEBSDV>=205 */
-#   if	FREEBSDV<210
+#  else	/* FREEBSDV>=2005 */
+#   if	FREEBSDV<2010
 #define	NCACHE_NXT	nc_lru.tqe_next	/* link in NCACHE */
-#   else	/* FREEBSDV>=210 */
+#   else	/* FREEBSDV>=2010 */
 #include <stddef.h>
 #define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
-#   endif	/* FREEBSDV<210 */
-#  endif	/* FREEBSDV<205 */
+#   endif	/* FREEBSDV<2010 */
+#  endif	/* FREEBSDV<2005 */
 
 #define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
 #define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
@@ -370,4 +520,9 @@ struct	namecache {
 #  endif	/* DEFINED(HASNCVPID) */
 # endif  /* defined(HASNCACHE) */
 
+#if	FREEBSDV>=5000
+#define	VNODE_VFLAG	v_iflag
+#define	NCACHE_VROOT	VV_ROOT
+#endif	/* FREEBSDV>=5000 */
+
 #endif	/* FREEBSD_LSOF_H */
