@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.21 2001/06/08 15:47:42 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.33 2004/12/30 18:41:31 abe Exp $";
 #endif
 
 
@@ -44,7 +44,7 @@ _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
-#if	FREEBSDV>=200
+#if	FREEBSDV>=2000
 # if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
 
@@ -74,7 +74,7 @@ getmemsz(pid)
 	}
 }
 # endif	/* defined(HASPROCFS) */
-#endif	/* FREEBSDV>=200 */
+#endif	/* FREEBSDV>=2000 */
 
 
 #if	defined(HASFDESCFS) && HASFDESCFS==1
@@ -123,6 +123,32 @@ lkup_dev_tty_again:
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
+#if	defined(HASKQUEUE)
+/*
+ * process_kqueue() -- process kqueue file
+ *
+ * Strictly speaking this function should appear in dfile.c, because it is
+ * a file processing function.  However, the Net and Open BSD sources don't
+ * require a dfile.c, so this is the next best location for the function.
+ */
+
+void
+process_kqueue(ka)
+	KA_T ka;			/* kqueue file structure address */
+{
+	struct kqueue kq;		/* kqueue structure */
+
+	(void) snpf(Lf->type, sizeof(Lf->type), "KQUEUE");
+	enter_dev_ch(print_kptr(ka, (char *)NULL, 0));
+	if (!ka || kread(ka, (char *)&kq, sizeof(kq)))
+	    return;
+	(void) snpf(Namech, Namechl, "count=%d, state=%#x", kq.kq_count,
+	    kq.kq_state);
+	enter_nm(Namech);
+}
+#endif	/* defined(HASKQUEUE) */
+
+
 /*
  * process_node() - process vnode
  */
@@ -132,44 +158,47 @@ process_node(va)
 	KA_T va;			/* vnode kernel space address */
 {
 	dev_t dev, rdev;
-	unsigned char devs = 0;
+	unsigned char devs;
 	unsigned char lt;
-	unsigned char rdevs = 0;
+	unsigned char rdevs;
 	char dev_ch[32], *ep;
-	struct inode *i = (struct inode *)NULL;
+	struct inode *i;
 	struct lockf lf, *lff, *lfp;
-	struct nfsnode *n = (struct nfsnode *)NULL;
+	struct nfsnode *n;
 	char *ty;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
-#if	FREEBSDV>=200
+#if	FREEBSDV>=2000
 	struct inode ib;
 	struct nfsnode nb;
 	size_t sz;
-# if	FREEBSDV>=400
+# if	FREEBSDV>=4000
+#  if	FREEBSDV<5000
 	struct specinfo si;
-# endif	/* FREEBSDV>=400 */
-#endif	/* FREEBSDV>=200 */
+#  else	/* FREEBSDV>=5000 */
+	struct cdev si;
+#  endif	/* FREEBSDV<5000 */
+# endif	/* FREEBSDV>=4000 */
+#endif	/* FREEBSDV>=2000 */
 
-#if	FREEBSDV<500
-	struct mfsnode *m = (struct mfsnode *)NULL;
-# if	FREEBSDV>=200
+#if	FREEBSDV<5000
+	struct mfsnode *m;
+# if	FREEBSDV>=2000
 	struct mfsnode mb;
-# endif	/* FREEBSDV>=200 */
-#endif	/* FREEBSDV<500 */
+# endif	/* FREEBSDV>=2000 */
+#endif	/* FREEBSDV<5000 */
 
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
-	int iso_dev_def = 0;
+	int iso_dev_def, iso_stat;
 	unsigned long iso_ino, iso_sz;
 	long iso_links;
-	int iso_stat = 0;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
-	struct fdescnode *f = (struct fdescnode *)NULL;
+	struct fdescnode *f;
 
 # if	HASFDESCFS==1
 	static dev_t f_tty_dev;
@@ -177,28 +206,97 @@ process_node(va)
 	static int f_tty_s = 0;
 # endif	/* HASFDESCFS==1 */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 	struct fdescnode fb;
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 #endif	/* defined(HASFDESCFS) */
 
-#if	FREEBSDV>=500
+#if	FREEBSDV>=5000
+	struct cdev cd;
+	int cds;
 	struct devfs_dirent de;
-	struct devfs_dirent *d = (struct devfs_dirent *)NULL;
-#endif	/* FREEBSDV>=500 */
+	struct devfs_dirent *d;
+	char vtbuf[32];
+	char *vtbp;
+	enum vtagtype { VT_DEVFS, VT_FDESC, VT_ISOFS, VT_PSEUDOFS, VT_NFS,
+			VT_NULL, VT_UFS, VT_UNKNOWN
+		      };
+#endif	/* FREEBSDV>=5000 */
+
+#if	defined(HASNULLFS)
+# if	!defined(HASPRINTDEV)
+	char dbuf[32];
+# endif	/* !defined(HASPRINTDEV) */
+	char *dp, *np, tbuf[1024];
+	struct null_node nu;
+	int sc = 0;
+#endif	/* defined(HASNULLFS) */
 
 #if	defined(HASPROCFS)
-	struct pfsnode *p = (struct pfsnode *)NULL;
+	struct pfsnode *p;
 	struct procfsid *pfi;
 	static int pgsz = -1;
 	struct vmspace vm;
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 	struct pfsnode pb;
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 #endif	/* defined(HASPROCFS) */
 
+# if	defined(HASPSEUDOFS) 
+	struct pfs_node pn;
+	struct pfs_node *pnp;
+# endif	/* defined(HASPSEUDOFS) */
+
+	enum vtagtype vtag;			/* placed here to use the
+						 * artificial vtagtype
+						 * definition required for
+						 * FREEBSDV>=5000 */
+
+#if	defined(HASNULLFS)
+
+process_overlaid_node:
+
+	if (++sc > 1024) {
+	    (void) snpf(Namech, Namechl, "too many overlaid nodes");
+	    enter_nm(Namech);
+	    return;
+	}
+#endif	/* defined(HASNULLFS) */
+
+/*
+ * Initialize miscellaneous variables.  This is done so that processing an
+ * overlaid node will be a fresh start.
+ */
+	devs = rdevs = 0;
+	i = (struct inode *)NULL;
+	n = (struct nfsnode *)NULL;
+	Namech[0] = '\0';
+
+#if	defined(HAS9660FS)
+	iso_dev_def = iso_stat = 0;
+#endif	/* defined(HAS9660FS) */
+
+#if	defined(HASFDESCFS)
+	f = (struct fdescnode *)NULL;
+#endif	/* defined(HASFDESCFS) */
+
+#if	FREEBSDV<5000
+	m = (struct mfsnode *)NULL;
+#else	/* FREEBSDV>=5000 */
+	d = (struct devfs_dirent *)NULL;
+	cds = 0;
+#endif	/* FREEBSDV<5000 */
+
+#if	defined(HASPROCFS)
+	p = (struct pfsnode *)NULL;
+#endif	/* defined(HASPROCFS) */
+
+# if	defined(HASPSEUDOFS) 
+	pnp = (struct pfs_node *)NULL;
+# endif	/* defined(HASPSEUDOFS) */
+
 /*
  * Read the vnode.
  */
@@ -214,9 +312,9 @@ process_node(va)
 
 #if	defined(HASNCACHE)
 	Lf->na = va;
-# if	defined(HASNCAPID)
+# if	defined(HASNCVPID)
 	Lf->id = v->v_id;
-# endif	/* defined(HASNCAPID) */
+# endif	/* defined(HASNCVPID) */
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASFSTRUCT)
@@ -243,8 +341,8 @@ process_node(va)
 		case MOUNT_PROCFS:
 		    Ntype = N_PROC;
 		    break;
-		}
 # endif	/* defined(HASPROCFS) */
+		}
 #else	/* !defined(MOUNT_NONE) */
 		if (strcasecmp(vfs->typnm, "nfs") == 0)
 		    Ntype = N_NFS;
@@ -253,6 +351,11 @@ process_node(va)
 		else if (strcasecmp(vfs->typnm, "procfs") == 0)
 		    Ntype = N_PROC;
 # endif	/* defined(HASPROCFS) */
+
+# if	defined(HASPSEUDOFS)
+		else if (strcasecmp(vfs->typnm, "pseudofs") == 0)
+		    Ntype = N_PSEU;
+# endif	/* defined(HASPSEUDOFS) */
 #endif	/* defined(MOUNT_NONE) */
 
 	    }
@@ -264,12 +367,55 @@ process_node(va)
 		break;
 	    }
 	}
+
+#if	FREEBSDV>=5000
+/*
+ * Get the FreeBSD 5 and above v_rdev structure for VCHR and VBLD vnodes.
+ */
+	if (((v->v_type == VCHR) || (v->v_type == VBLK))
+	&&  v->v_rdev
+	&&  !kread((KA_T)v->v_rdev, (char *)&cd, sizeof(cd))
+	) {
+	    cds = 1;
+	}
+#endif	/* FREEBSDV>=5000 */
+
 /*
  * Define the specific node pointer.
  */
-	switch (v->v_tag) {
 
-#if	FREEBSDV>=500
+#if	FREEBSDV>=5000
+/*
+ * Get the pseudo vnode tag type for FreeBSD >= 5.
+ */
+	vtag = VT_UNKNOWN;
+	if (v->v_tag && !kread((KA_T)v->v_tag, (char *)&vtbuf, sizeof(vtbuf)))
+	{
+	    vtbuf[sizeof(vtbuf) - 1] = '\0';
+	    vtbp = vtbuf;
+	    if (!strcmp(vtbuf, "ufs"))
+		vtag = VT_UFS;
+	    else if (!strcmp(vtbuf, "devfs"))
+		vtag = VT_DEVFS;
+	    else if (!strcmp(vtbuf, "nfs"))
+		vtag = VT_NFS;
+	    else if (!strcmp(vtbuf, "isofs"))
+		vtag = VT_ISOFS;
+	    else if (!strcmp(vtbuf, "pseudofs"))
+		vtag = VT_PSEUDOFS;
+	    else if (!strcmp(vtbuf, "null"))
+		vtag = VT_NULL;
+	    else if (!strcmp(vtbuf, "fdesc"))
+		vtag = VT_FDESC;
+	} else
+	    vtbp = "(unknown)";
+#else	/* FREEBSDV<5000 */
+	vtag = v->v_tag;
+#endif	/* FREEBSDV>=5000 */
+
+	switch (vtag) {
+
+#if	FREEBSDV>=5000
 	case VT_DEVFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&de, sizeof(de)))
@@ -290,7 +436,24 @@ process_node(va)
 		}
 	    }
 	    break;
-#endif	/* FREEBSDV>=500 */
+#endif	/* FREEBSDV>=5000 */
+
+#if	defined(HASFDESCFS)
+	case VT_FDESC:
+
+# if	FREEBSDV<2000
+	    f = (struct fdescnode *)v->v_data;
+# else	/* FREEBSDV>=2000 */
+	    if (kread((KA_T)v->v_data, (char *)&fb, sizeof(fb)) != 0) {
+		(void) snpf(Namech, Namechl, "can't read fdescnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    f = &fb;
+	    break;
+# endif	/* FREEBSDV<2000 */
+#endif	/* defined(HASFDESCFS) */
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
@@ -306,12 +469,12 @@ process_node(va)
 	    break;
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV<500
+#if	FREEBSDV<5000
 	case VT_MFS:
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 	    m = (struct mfsnode *)v->v_data;
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&mb, sizeof(mb))) {
 		(void) snpf(Namech, Namechl, "no mfs node: %s",
@@ -320,15 +483,15 @@ process_node(va)
 		return;
 	    }
 	    m = &mb;
-# endif	/* FREEBSDV<200 */
-#endif	/* FREEBSDV<500 */
+# endif	/* FREEBSDV<2000 */
+#endif	/* FREEBSDV<5000 */
 
 	    break;
 	case VT_NFS:
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	    n = (struct nfsnode *)v->v_data;
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&nb, sizeof(nb))) {
 		(void) snpf(Namech, Namechl, "no nfs node: %s",
@@ -337,33 +500,70 @@ process_node(va)
 		return;
 	    }
 	    n = &nb;
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	    break;
 
-#if	defined(HASFDESCFS)
-	case VT_FDESC:
+#if	defined(HASNULLFS)
+	case VT_NULL:
+	    if (sc == 1) {
 
-# if	FREEBSDV<200
-	    f = (struct fdescnode *)v->v_data;
-# else	/* FREEBSDV>=200 */
-	    if (kread((KA_T)v->v_data, (char *)&fb, sizeof(fb)) != 0) {
-		(void) snpf(Namech, Namechl, "can't read fdescnode at: %s",
+	    /*
+	     * If this is the first null_node, enter a name addition containing
+	     * the mounted-on directory, the file system name, and the device
+	     * number.
+	     */
+		if (vfs && (vfs->dir || vfs->fsname || vfs->fsid.val[0])) {
+		    if (vfs->fsid.val[0]) {
+
+#if	defined(HASPRINTDEV)
+			dp = HASPRINTDEV(Lf, &dev);
+#else	/* !defined(HASPRINTDEV) */
+			(void) snpf(dbuf, sizeof(dbuf) - 1, "%d,%d",
+			    GET_MAJ_DEV(dev), GET_MIN_DEV(dev));
+			dbuf[sizeof(dbuf) - 1] = '\0';
+			dp = dbuf;
+#endif	/* defined(HASPRINTDEV) */
+
+		    } else
+			dp = (char *)NULL;
+		    (void) snpf(tbuf, sizeof(tbuf) - 1,
+			"(nullfs%s%s%s%s%s%s%s)",
+			(vfs && vfs->fsname) ? " " : "",
+			(vfs && vfs->fsname) ? vfs->fsname : "",
+			(vfs && vfs->dir) ? " on " : "",
+			(vfs && vfs->dir) ? vfs->dir : "",
+			(dp && vfs && vfs->dir) ? " (" : "",
+			(dp && vfs && vfs->dir) ? dp : "",
+			(dp && vfs && vfs->dir) ? ")" : "");
+		    tbuf[sizeof(tbuf) - 1] = '\0';
+		    np = tbuf;
+		} else
+		    np = "(nullfs)";
+		(void) add_nma(np, (int)strlen(np));
+	    }
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&nu, sizeof(nu))) {
+		(void) snpf(Namech, Namechl, "can't read null_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
-	    f = &fb;
-	    break;
-# endif	/* FREEBSDV<200 */
-#endif	/* defined(HASFDESCFS) */
+	    if (!nu.null_lowervp) {
+		(void) snpf(Namech, Namechl, "null_node overlays nothing");
+		enter_nm(Namech);
+		return;
+	    }
+	    va = (KA_T)nu.null_lowervp;
+	    goto process_overlaid_node;
+#endif	/* defined(HASNULLFS) */
 
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 	    p = (struct pfsnode *)v->v_data;
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&pb, sizeof(pb))) {
 		(void) snpf(Namech, Namechl, "no pfs node: %s",
@@ -372,16 +572,29 @@ process_node(va)
 		return;
 	    }
 	    p = &pb;
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPSEUDOFS)
+	case VT_PSEUDOFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&pn, sizeof(pn))) {
+		(void) snpf(Namech, Namechl, "no pfs_node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    pnp = &pn;
+	    break;
+#endif	/* defined(HASPSEUDOFS) */
+
 	case VT_UFS:
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	    i = (struct inode *)v->v_data;
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&ib, sizeof(ib))) {
 		(void) snpf(Namech, Namechl, "no ufs node: %s",
@@ -390,7 +603,7 @@ process_node(va)
 		return;
 	    }
 	    i = &ib;
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	    if ((lff = i->i_lockf)) {
 
@@ -436,7 +649,13 @@ process_node(va)
 	default:
 	    if (v->v_type == VBAD || v->v_type == VNON)
 		break;
+
+#if	FREEBSDV<5000
 	    (void) snpf(Namech,Namechl,"unknown file system type: %d",v->v_tag);
+#else	/* FREEBSDV>=5000 */
+	    (void) snpf(Namech, Namechl, "unknown file system type: %s", vtbp);
+#endif	/* FREEBSDV<5000 */
+
 	    enter_nm(Namech);
 	    return;
 	}
@@ -453,20 +672,29 @@ process_node(va)
 	    }
 	} else if (i) {
 
-#if	FREEBSDV>=400
+#if	FREEBSDV>=4000
 	    if (i->i_dev
 	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))) {
 		dev = si.si_udev;
 		devs = 1;
 	    }
-#else	/* FREEBSDV<400 */
+#else	/* FREEBSDV<4000 */
 	    dev = i->i_dev;
 	    devs = 1;
-#endif	/* FREEBSDV>=400 */
+#endif	/* FREEBSDV>=4000 */
 
 	    if ((type == VCHR) || (type == VBLK)) {
-		rdev = i->i_rdev ;
+
+#if	FREEBSDV>=5000
+		if (cds) {
+		    rdev = cd.si_udev;
+		    rdevs = 1;
+		}
+#else	/* FREEBSDV<5000 */
+		rdev = i->i_rdev;
 		rdevs = 1;
+#endif	/* FREEBSDV>=5000 */
+
 	    }
 	}
 
@@ -496,7 +724,7 @@ process_node(va)
 # endif	/* HASFDESFS==1 */
 
 	}
-#endif	defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1)
+#endif	/* defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1) */
 
 #if	defined(HAS9660FS)
 	else if (iso_stat && iso_dev_def) {
@@ -505,7 +733,7 @@ process_node(va)
 	}
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV>=500
+#if	FREEBSDV>=5000
 	else if (d) {
 	    if (vfs) {
 		dev = vfs->fsid.val[0];
@@ -514,23 +742,28 @@ process_node(va)
 		dev = DevDev;
 		devs = 1;
 	    }
-	    if ((type == VCHR) && v->v_rdev) {
-		if (!kread((KA_T)v->v_rdev, (char *)&si, sizeof(si))) {
-		    rdev = si.si_udev;
-		    rdevs = 1;
-		}
+	    if ((type == VCHR) && cds) {
+		rdev = cd.si_udev;
+		rdevs = 1;
 	    }
 	}
-#endif	/* FREEBSDV>=500 */
+#endif	/* FREEBSDV>=5000 */
+
+#if	defined(HASPSEUDOFS)
+	else if (pnp) {
+	    if (vfs) {
+		dev = vfs->fsid.val[0];
+		devs = 1;
+	    }
+	}
+#endif	/* defined(HASPSEUDOFS) */
 
 /*
  * Obtain the inode number.
  */
 	if (i) {
-	    if (type != VBLK) {
-		Lf->inode = (unsigned long)i->i_number;
-		Lf->inp_ty = 1;
-	    }
+	    Lf->inode = (unsigned long)i->i_number;
+	    Lf->inp_ty = 1;
 	} else if (n) {
 	    Lf->inode = (unsigned long)n->n_vattr.va_fileid;
 	    Lf->inp_ty = 1;
@@ -544,21 +777,27 @@ process_node(va)
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASPROCFS)
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 	else if (p) {
 	    Lf->inode = (unsigned long)p->pfs_fileno;
 	    Lf->inp_ty = 1;
 	}
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 #endif	/* defined(HASPROCFS) */
 
-#if	FREEBSDV>=500
+#if	defined(HASPSEUDOFS)
+	else if (pnp) {
+	    Lf->inode = (unsigned long)pnp->pn_fileno;
+	    Lf->inp_ty = 1;
+	}
+#endif	/* defined(HASPSEUDOFS) */
+
+#if	FREEBSDV>=5000
 	else if (d) {
 	    Lf->inode = (unsigned long)d->de_inode;
 	    Lf->inp_ty = 1;
 	}
-#endif	/* FREEBSDV>=500 */
-
+#endif	/* FREEBSDV>=5000 */
 
 /*
  * Obtain the file size.
@@ -581,7 +820,7 @@ process_node(va)
 #if	defined(HASPROCFS)
 	    case N_PROC:
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 		if (type == VDIR || !p || !p->pfs_vs
 		||  kread((KA_T)p->pfs_vs, (char *)&vm, sizeof(vm)))
 		    break;
@@ -592,7 +831,7 @@ process_node(va)
 		       +         (pgsz * vm.vm_ssize));
 		Lf->sz_def = 1;
 		break;
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 		if (p) {
 		    switch(p->pfs_type) {
 		    case Proot:
@@ -613,9 +852,16 @@ process_node(va)
 			break;
 		    }
 		}
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPSEUDOFS)
+	    case N_PSEU:
+		Lf->sz = 0;
+		Lf->sz_def = 1;
+		break;
+#endif	/* defined(PSEUDOFS) */
+
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (i) {
@@ -623,12 +869,12 @@ process_node(va)
 			Lf->sz_def = 1;
 		    }
 
-#if	FREEBSDV<500
+#if	FREEBSDV<5000
 		    else if (m) {
 			Lf->sz = (SZOFFTYPE)m->mfs_size;
 			Lf->sz_def = 1;
 		    }
-#endif	/* FREEBSDV<500 */
+#endif	/* FREEBSDV<5000 */
 
 #if	defined(HAS9660FS)
 		    else if (iso_stat) {
@@ -656,7 +902,13 @@ process_node(va)
 		break;
 	    case N_REGLR:
 		if (i) {
+
+#if	defined(HASEFFNLINK)
+		    Lf->nlink = (long)i->HASEFFNLINK;
+#else	/* !defined(HASEFFNLINK) */
 		    Lf->nlink = (long)i->i_nlink;
+#endif	/* defined(HASEFFNLINK) */
+
 		    Lf->nlink_def = 1;
 		}
 
@@ -667,15 +919,24 @@ process_node(va)
 		}
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV>=500
+#if	FREEBSDV>=5000
 		else if (d) {
 		    Lf->nlink = d->de_links;
 		    Lf->nlink_def = 1;
 		}
-#endif	/* FREEBSDV>=500 */
-
+#endif	/* FREEBSDV>=5000 */
 
 		break;
+
+#if	defined(HASPSEUODOFS)
+	    case N_PSEU:
+		if (pnp) {
+		    Lf->nlink = 1L;
+		    Lf->nlink_def = 1;
+		}
+		break;
+#endif	/* defined(HASPSEUODOFS) */
+
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
 		Lf->sf |= SELNLINK;
@@ -755,28 +1016,28 @@ process_node(va)
 	if (type == VBAD)
 	    (void) snpf(Namech, Namechl, "(revoked)");
 
-#if	FREEBSDV<500
+#if	FREEBSDV<5000
 	else if (m) {
 	    Lf->dev_def = Lf->rdev_def = 0;
 	    (void) snpf(Namech, Namechl, "%#x", m->mfs_baseoff);
 	    (void) snpf(dev_ch, sizeof(dev_ch), "    memory");
 	    enter_dev_ch(dev_ch);
 	}
-#endif	/* FREEBSDV<500 */
+#endif	/* FREEBSDV<5000 */
 
 
 #if	defined(HASPROCFS)
 	else if (p) {
 	    Lf->dev_def = Lf->rdev_def = 0;
 
-# if	FREEBSDV<200
+# if	FREEBSDV<2000
 	    if (type == VDIR)
 		(void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    else
 		(void) snpf(Namech, Namechl, "/%s/%0*d", HASPROCFS, PNSIZ,
 		    p->pfs_pid);
 	    enter_nm(Namech);
-# else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=2000 */
 	    ty = (char *)NULL;
 	    (void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    switch (p->pfs_type) {
@@ -829,7 +1090,7 @@ process_node(va)
 		ty = "PGID";
 		break;
 
-#  if	FREEBSDV>=300
+#  if	FREEBSDV>=3000
 	    case Pmap:
 		ep = endnm(&sz);
 		(void) snpf(ep, sz, "/%d/map", p->pfs_pid);
@@ -840,14 +1101,14 @@ process_node(va)
 		(void) snpf(ep, sz, "/%d/etype", p->pfs_pid);
 		ty = "PETY";
 		break;
-#  endif	/* FREEBSDV>=300 */
+#  endif	/* FREEBSDV>=3000 */
 
 	    }
 	    if (ty)
 		(void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	    enter_nm(Namech);
 
-# endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<2000 */
 	}
 #endif	/* defined(HASPROCFS) */
 
@@ -911,7 +1172,7 @@ process_node(va)
 }
 
 
-#if	FREEBSDV>=220
+#if	FREEBSDV>=2020
 /*
  * process_pipe() - process a file structure whose type is DTYPE_PIPE
  */
@@ -963,4 +1224,4 @@ process_pipe(pa)
 	if (Namech[0])
 	    enter_nm(Namech);
 }
-#endif	/* FREEBSDV>=220 */
+#endif	/* FREEBSDV>=2020 */
