@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.15 2001/01/02 12:34:19 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.25 2004/07/06 19:18:57 abe Exp $";
 #endif
 
 
@@ -65,7 +65,6 @@ process_socket(sa)
 	int fp, lp;
 	struct inpcb inp;
 	unsigned char *la = (unsigned char *)NULL;
-	struct mbuf mb;
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
@@ -74,6 +73,12 @@ process_socket(sa)
 	struct sockaddr_un *ua = NULL;
 	struct sockaddr_un un;
 
+#if	FREEBSDV<4050
+	struct mbuf mb;
+#else	/* FREEBSDV>=4050 */
+	int unl;
+#endif	/* FREEBSDV<4050 */
+
 #if	defined(HASIPv6) && !defined(HASINRIAIPv6)
 	struct in6pcb in6p;
 #endif	/* defined(HASIPv6) && !defined(HASINRIAIPv6) */
@@ -131,6 +136,25 @@ process_socket(sa)
 	Lf->lts.rqs = Lf->lts.sqs = 1;
 #endif	/* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+	Lf->lts.ltm = (unsigned int)s.so_linger;
+	Lf->lts.opt = (unsigned int)s.so_options;
+	Lf->lts.qlen = (unsigned int)s.so_qlen;
+	Lf->lts.qlim = (unsigned int)s.so_qlimit;
+	Lf->lts.rbsz = (unsigned long)s.so_rcv.sb_mbmax;
+	Lf->lts.sbsz = (unsigned long)s.so_snd.sb_mbmax;
+	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		      = Lf->lts.sbszs = (unsigned char)1;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	Lf->lts.ss = (unsigned int)s.so_state;
+# if	defined(HASSBSTATE)
+	Lf->lts.sbs_rcv = s.so_rcv.sb_state;
+	Lf->lts.sbs_snd = s.so_snd.sb_state;
+# endif	/* defined(HASSBSTATE) */
+#endif	/* defined(HASSOSTATE) */
+
 /*
  * Process socket by the associated domain family.
  */
@@ -144,8 +168,17 @@ process_socket(sa)
 	case AF_INET6:
 #endif	/* defined(HASIPv6) */
 
-	    if (Fnet)
-		Lf->sf |= SELNET;
+	    if (Fnet) {
+		if (!FnetTy
+		||  ((FnetTy == 4) && (fam == AF_INET))
+
+#if	defined(HASIPv6)
+		||  ((FnetTy == 6) && (fam == AF_INET6))
+#endif	/* defined(HASIPv6) */
+		)
+
+		    Lf->sf |= SELNET;
+	    }
 	    printiproto(p.pr_protocol);
 
 #if	defined(HASIPv6)
@@ -196,10 +229,21 @@ process_socket(sa)
 		||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
 		    if (!s.so_pcb) {
 			(void) snpf(Namech, Namechl, "no PCB%s%s",
-			    (s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE"
-							   : "",
-			    (s.so_state & SS_CANTRCVMORE) ? ", CANTRCVMORE"
-							  : "");
+
+#if	defined(HASSBSTATE)
+			    (s.so_snd.sb_state & SBS_CANTSENDMORE) ?
+#else	/* !defined(HASSBSTATE) */
+			    (s.so_state & SS_CANTSENDMORE) ?
+#endif	/* defined(HASSBSTATE) */ 
+
+				", CANTSENDMORE" : "",
+#if	defined(HASSBSTATE)
+			    (s.so_rcv.sb_state & SBS_CANTRCVMORE) ?
+#else	/* !defined(HASSBSTATE) */
+			    (s.so_state & SS_CANTRCVMORE) ?
+#endif	/* defined(HASSBSTATE) */
+
+				", CANTRCVMORE" : "");
 		    } else {
 			(void) snpf(Namech, Namechl, "can't read inpcb at %s",
 			    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
@@ -261,10 +305,16 @@ process_socket(sa)
  	 */
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
-	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+	    if (ta && !kread(ta, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)t.t_state;
+
+#if	defined(HASTCPOPT)
+		Lf->lts.mss = (unsigned long)t.t_maxseg;
+		Lf->lts.msss = (unsigned char)1;
+		Lf->lts.topt = (unsigned int)t.t_flags;
+#endif	/* defined(HASTCPOPT) */
+
 	    }
 	    break;
 /*
@@ -302,14 +352,27 @@ process_socket(sa)
 		break;
 	    }
 	    if (unp.unp_addr) {
-		if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
+
+#if	FREEBSDV<4050
+		if (kread((KA_T)unp.unp_addr, (char *)&mb, sizeof(mb)))
+#else	/* FREEBSDV>=4050 */
+		if (kread((KA_T)unp.unp_addr, (char *)&un, sizeof(un)))
+#endif	/* FREEBSDV<4050 */
+
+		{
 		    (void) snpf(Namech, Namechl, "can't read unp_addr at %s",
 			print_kptr((KA_T)unp.unp_addr, (char *)NULL, 0));
 		    break;
 		}
+
+#if	FREEBSDV<4050
 		if (mb.m_hdr.mh_len == sizeof(struct sockaddr_un))
 		    ua = (struct sockaddr_un *) ((char *) &mb
 		       + (mb.m_hdr.mh_data - (caddr_t) unp.unp_addr));
+#else	/* FREEBSDV>=4050 */
+		ua = &un;
+#endif	/* FREEBSDV<4050 */
+
 	    }
 	    if (!ua) {
 		ua = &un;
@@ -339,12 +402,25 @@ process_socket(sa)
 		break;
 	    }
 	    if (ua->sun_path[0]) {
+
+#if	FREEBSDV<4050
 		if (mb.m_len >= sizeof(struct sockaddr_un))
 		    mb.m_len = sizeof(struct sockaddr_un) - 1;
 		*((char *)ua + mb.m_len) = '\0';
-		if (Sfile && is_file_named(ua->sun_path, 0))
+#else	/* FREEBSDV>=4050 */
+# if	FREEBSDV>4060
+		unl = ua->sun_len - offsetof(struct sockaddr_un, sun_path);
+# else	/* FREEBSDV<4060 */
+		unl = sizeof(ua->sun_path) - 1;
+# endif	/* FREEBSDV>4060 */ 
+		if ((unl < 0) || (unl >= sizeof(ua->sun_path)))
+		    unl = sizeof(ua->sun_path) - 1;
+		ua->sun_path[unl] = '\0';
+#endif	/* FREEBSDV<4050 */
+
+		if (ua->sun_path[0] && Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
-		if (!Namech[0])
+		if (ua->sun_path[0] && !Namech[0])
 		    (void) snpf(Namech, Namechl, "%s", ua->sun_path);
 	    } else
 		(void) snpf(Namech, Namechl, "no address");
