@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.27 2001/06/06 17:59:14 abe Exp $
+ * $Id: machine.h,v 1.33 2004/10/17 21:55:01 abe Exp $
  */
 
 
@@ -40,6 +40,23 @@
 
 
 #include <sys/types.h>
+
+#if	defined(HASCPUMASK_T)
+/*
+ * In FreeBSD >= 5.2 when the cpumask_t typedef is present, it is defined int
+ * <sys/types.h> only if _KERNEL is predefined.  However, predefining _KERNEL
+ * before #include'ing <sys/types.h> causes redefinition errors for boolean_t
+ * and vm_page_t when <vm/vm.h> is #include'd with _KERNEL predefined.  Since
+ * lsof must have _KERNEL predefined when <vm/vm.h> is #include'd, the
+ * expedient choice is made to duplicate the cpumask_t typedef here.
+ *
+ * Note: the Configure script defines HASCPUMASK_T if <sys/types.h> and
+ * <machine/_types.h> both have cpumask_t references.
+ */
+
+typedef	__cpumask_t	cpumask_t;
+#endif	/* defined(HASCPUMASK_T) */
+
 #include <sys/param.h>
 
 
@@ -51,6 +68,14 @@
 #define	CAN_USE_CLNT_CREATE	1
 
 
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+#define	DEVDEV_PATH	"/dev"
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -73,9 +98,9 @@
  * recorded in BDevtp[].
  */
 
-# if	FREEBSDV<400
+# if	FREEBSDV<4000
 #define	HASBLKDEV	1
-# endif	/* FREEBSDV<400 */
+# endif	/* FREEBSDV<4000 */
 
 
 /*
@@ -203,6 +228,14 @@
 /* #define SETLFILEADD Lf->... */
 
 
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+/* #define	HASMNTSUP	1	*/
+
+
 /*
  * HASMOPT is defined for those dialects that support the reading of
  * kernel memory from an alternate file.
@@ -216,16 +249,12 @@
  * that lsof can search.  A value of 1 directs printname() to prefix the
  * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
- *
  * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
  *
  * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
-#define	HASNCAPID	1
 /* #define	NCACHELDPFX	??? */
 /* #define	NCACHELDSFX	??? */
 
@@ -246,9 +275,9 @@
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-# if	FREEBSDV>=220
+# if	FREEBSDV>=2020
 #define	HASPIPEFN	process_pipe
-# endif	/* FREEBSDV>=220 */
+# endif	/* FREEBSDV>=2020 */
 
 
 /*
@@ -271,9 +300,9 @@
  * the parent process IDentifier (PPID) of a process.
  */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 #define	HASPPID		1
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 
 /*
@@ -360,9 +389,9 @@
 /* #define		HASPROCFS	"proc?" */
 /* #define		HASFSTYPE	1 */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 #define	HASPINODEN	1
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 
 /*
@@ -382,13 +411,33 @@
 /* #define	HASSECURITY	1 */
 
 
+/*
+ * If HASSECURITY is defined, define HASNOSOCKSECURITY to allow users
+ * restricted by HASSECURITY to list any open socket files, provide their
+ * listing is selected by the "-i" option.
+ */
+
+/* #define	HASNOSOCKSECURITY	1	*/
+
+
 /*
  * HASSETLOCALE is defined for those dialects that have <locale.h> and
  * setlocale().
+ *
+ * If the dialect also has wide character support for language locales,
+ * HASWIDECHAR activates lsof's wide character support and WIDECHARINCL
+ * defines the header file (if any) that must be #include'd to use the
+ * mblen() and mbtowc() functions.
  */
 
 #define	HASSETLOCALE	1
 
+# if	FREEBSDV>=5200
+#define	HASWIDECHAR	1
+# endif	/* FREEBSDV>=5020 */
+
+/* #define	WIDECHARINCL	<wchar.h>	*/
+
 
 /*
  * HASSNODE is defined for those dialects that have snodes.
@@ -397,6 +446,17 @@
 /* #define	HASSNODE	1 */
 
 
+/*
+ * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
+ * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
+ * options.
+ */
+
+#define	HASSOOPT	1	/* has socket option information */
+#define	HASSOSTATE	1	/* has socket state information */
+#define	HASTCPOPT	1	/* has TCP options or flags */
+
+
 /*
  * Define HASSPECDEVD to be the name of a function that handles the results
  * of a successful stat(2) of a file name argument.
@@ -487,18 +547,18 @@
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 #define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
-#define	USE_LIB_PRINTCHDEVNAME			1	/* pdvn.c */
+#define	USE_LIB_PRINTDEVNAME			1	/* pdvn.c */
 #define	USE_LIB_PROCESS_FILE			1	/* prfp.c */
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
 #define	USE_LIB_READDEV				1	/* rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
 /* #define	USE_LIB_REGEX			1	   regex.c */
 
-# if	FREEBSDV<210
+# if	FREEBSDV<2010
 #define	USE_LIB_RNAM				1	/* rnam.c */
-# else	/* FREEBSDV>=210 */
+# else	/* FREEBSDV>=2010 */
 #define	USE_LIB_RNMH				1	/* rnmh.c */
-# endif	/* FREEBSDV<210 */
+# endif	/* FREEBSDV<2010 */
 
 /* #define	USE_LIB_RNCH			1	   rnch.c */
 /* #define	USE_LIB_SNPF			1	   snpf.c */
