@@ -1,12 +1,12 @@
 /*
- * dproto.h - Linux function prototypes for lsof
+ * dproto.h - Linux function prototypes for /proc-based lsof
  *
  * The _PROTOTYPE macro is defined in the common proto.h.
  */
 
 
 /*
- * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
+ * Copyright 1997 Purdue Research Foundation, West Lafayette, Indiana
  * 47907.  All rights reserved.
  *
  * Written by Victor A. Abell
@@ -33,11 +33,15 @@
 
 
 /*
- * $Id: dproto.h,v 1.2 96/01/09 08:18:28 abe Exp $
+ * $Id: dproto.h,v 1.3 2004/07/06 19:19:44 abe Exp $
  */
 
 
 
-_PROTOTYPE(extern int is_file_named,(char *p, umode_t ty));
-_PROTOTYPE(extern void process_inode,(struct inode *fi, struct file_operations *fop));
-_PROTOTYPE(extern void process_isock,(struct inode *i));
+_PROTOTYPE(extern int get_fields,(char *ln, char *sep, char ***fr));
+_PROTOTYPE(extern void get_locks,(char *p));
+_PROTOTYPE(extern void get_net,(char *p, int pl));
+_PROTOTYPE(extern int is_file_named,(char *p, int cd));
+_PROTOTYPE(extern int make_proc_path,(char *pp, int lp, char **np, int *npl, char *sf));
+_PROTOTYPE(extern void process_proc_node,(char *p, struct stat *s, int ss, struct stat *l, int ls));
+_PROTOTYPE(extern void process_proc_sock,(char *p, struct stat *s, int ss, struct stat *l, int ls));
