@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.18 2001/01/02 12:34:33 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.27 2004/12/30 18:41:59 abe Exp $";
 #endif
 
 
@@ -87,7 +87,7 @@ struct tcp_udp {			/* IPv4 TCP and UDP socket
 	unsigned long inode;
 	unsigned long faddr, laddr;	/* foreign & local IPv6 addresses */
 	int fport, lport;		/* foreign & local ports */
-	unsigned long txq, rxq;		/* trasmit & receve queue values */
+	unsigned long txq, rxq;		/* transmit & receive queue values */
 	int proto;			/* 0 = TCP, 1 = UDP */
 	int state;			/* protocol state */
 	struct tcp_udp *next;
@@ -99,7 +99,7 @@ struct tcp_udp6 {			/* IPv6 TCP and UDP socket
 	unsigned long inode;
 	struct in6_addr faddr, laddr;	/* foreign and local IPv6 addresses */
 	int fport, lport;		/* foreign & local ports */
-	unsigned long txq, rxq;		/* trasmit & receve queue values */
+	unsigned long txq, rxq;		/* transmit & receive queue values */
 	int proto;			/* 0 = TCP, 1 = UDP */
 	int state;			/* protocol state */
 	struct tcp_udp6 *next;
@@ -110,6 +110,10 @@ struct uxsin {				/* UNIX socket information */
 	unsigned long inode;
 	char *pcb;
 	char *path;
+	unsigned char sb_def;
+	dev_t sb_dev;
+	unsigned long sb_ino;
+	dev_t sb_rdev;
 	struct uxsin *next;
 };
 
@@ -120,6 +124,14 @@ struct uxsin {				/* UNIX socket information */
 
 static struct ax25sin **AX25sin = (struct ax25sin **)NULL;
 					/* AX25 socket info, hashed by inode */
+static char *ax25st[] = {
+	"LISTENING",			/* 0 */
+	"SABM SENT",			/* 1 */
+	"DISC SENT",			/* 2 */
+	"ESTABLISHED",			/* 3 */
+	"RECOVERY"			/* 4 */
+};
+#define NAX25ST	(sizeof(ax25st) / sizeof(char *))
 static struct ipxsin **Ipxsin = (struct ipxsin **)NULL;
 					/* IPX socket info, hashed by inode */
 static struct rawsin **Rawsin = (struct rawsin **)NULL;
@@ -129,6 +141,9 @@ static struct tcp_udp **TcpUdp = (struct tcp_udp **)NULL;
 					 * inode */
 
 #if	defined(HASIPv6)
+static struct rawsin **Rawsin6 = (struct rawsin **)NULL;
+					/* IPv6 raw socket info, hashed by
+					 * inode */
 static struct tcp_udp6 **TcpUdp6 = (struct tcp_udp6 **)NULL;
 					/* IPv6 TCP & UDP info, hashed by
 					 * inode */
@@ -157,17 +172,16 @@ _PROTOTYPE(static void print_ipxinfo,(struct ipxsin *ip));
 _PROTOTYPE(static void print_rawinfo,(struct rawsin *ip));
 
 #if	defined(HASIPv6)
-_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
+_PROTOTYPE(static void get_raw6,(char *p));
 _PROTOTYPE(static void get_tcpudp6,(char *p, int pr, int clr));
 _PROTOTYPE(static int net6a2in6,(char *as, struct in6_addr *ad));
+_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
 #endif	/* defined(HASIPv6) */
 
 
 
 /*
  * check_ax25() - check for AX25 socket file
- *
- * DEBUG: this code hasn't been tested!
  */
 
 static struct ax25sin *
@@ -296,8 +310,6 @@ check_unix(i)
 
 /*
  * get_ax25() - get /proc/net/ax25 info
- *
- * DEBUG: this code hasn't been tested!
  */
 
 static void
@@ -307,7 +319,6 @@ get_ax25(p)
 	struct ax25sin *ap, *np;
 	FILE *as;
 	char buf[MAXPATHLEN], *da, *dev_ch, *ep, **fp, *sa;
-	int fl = 1;
 	int h, i, nf;
 	unsigned long inode, rq, sq, state;
 	MALLOC_S len;
@@ -346,49 +357,30 @@ get_ax25(p)
 	if (!(as = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, as)) {
-	    if ((nf = get_fields(buf, (char *)NULL, &fp)) < 19)
+	    if ((nf = get_fields(buf, (char *)NULL, &fp)) < 24)
 		continue;
-	    if (fl) {
-		fl = 0;
+	/*
+	 * /proc/next/ac25 has no title line, a very poor deficiency in its
+	 * implementation.
+	 *
+	 * The ax25_get_info() function in kern mnodule .../net/ax25/af_ax25.c
+	 * says the format of the lines in the file is:
+	 *
+	 *     magic dev src_addr dest_addr,digi1,digi2,.. st vs vr va t1 t1 \
+	 *     t2  t2 t3 t3 idle idle n2 n2 rtt window paclen Snd-Q Rcv-Q \
+	 *     inode
+	 *
+	 * The code in this function is forced to assume that format is in
+	 * effect..
+	 */
 
-	    /*
-	     * Check the column labels in the first line.
-	     */
-		if (!fp[0]  || strcmp(fp[0],  "dest_addr")
-		||  !fp[1]  || strcmp(fp[1],  "src_addr")
-		||  !fp[2]  || strcmp(fp[2],  "dev")
-		||  !fp[3]  || strcmp(fp[3],  "st")
-		||  !fp[4]  || strcmp(fp[4],  "vs")
-		||  !fp[5]  || strcmp(fp[5],  "vr")
-		||  !fp[6]  || strcmp(fp[6],  "va")
-		||  !fp[7]  || strcmp(fp[7],  "t1")
-		||  !fp[8]  || strcmp(fp[8],  "t2")
-		||  !fp[9]  || strcmp(fp[9],  "t3")
-		||  !fp[10] || strcmp(fp[10], "idle")
-		||  !fp[11] || strcmp(fp[11], "n2")
-		||  !fp[12] || strcmp(fp[12], "rtt")
-		||  !fp[13] || strcmp(fp[13], "wnd")
-		||  !fp[14] || strcmp(fp[14], "packlen")
-		||  !fp[15] || strcmp(fp[15], "dama")
-		||  !fp[16] || strcmp(fp[16], "Snd-Q")
-		||  !fp[17] || strcmp(fp[17], "Rcv-Q")
-		||  !fp[18] || strcmp(fp[18], "inode"))
-		{
-		    if (!Fwarn) {
-			(void) fprintf(stderr,
-			    "%s: WARNING: unsupported format: %s\n",
-			    Pn, p);
-		    }
-		    break;
-		}
-		continue;
-	    }
 	/*
 	 * Assemble the inode number and see if it has already been recorded.
+	 * If it has, skip this line.
 	 */
 	    ep = (char *)NULL;
-	    if (!fp[18] || !*fp[18]
-	    ||  (inode = strtoul(fp[18], &ep, 0)) == ULONG_MAX
+	    if (!fp[23] || !*fp[23]
+	    ||  (inode = strtoul(fp[23], &ep, 0)) == ULONG_MAX
 	    ||  !ep || *ep)
 		continue;
 	    h = INOHASH(inode);
@@ -403,64 +395,65 @@ get_ax25(p)
 	 */
 	    rq = sq = (unsigned long)0;
 	    rqs = sqs = (unsigned char)0;
-	    if (nf >= 19) {
-	 	p = (char *)NULL;
-		if (!fp[16] || !*fp[16]
-		||  (sq = strtoul(fp[16], &ep, 0)) == ULONG_MAX || !ep || *ep)
-		    continue;
-		sqs = (unsigned char)1;
-		ep = (char *)NULL;
-		if (!fp[17] || !*fp[17]
-		||  (rq = strtoul(fp[17], &ep, 0)) == ULONG_MAX || !ep || *ep)
-		    continue;
-		rqs = (unsigned char)1;
-	    }
 	    ep = (char *)NULL;
-	    if (!fp[3] || !*fp[3]
-	    ||  (state = strtoul(fp[3], &ep, 0)) == ULONG_MAX || !ep || *ep)
+	    if (!fp[21] || !*fp[21]
+	    ||  (sq = strtoul(fp[21], &ep, 0)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    sqs = (unsigned char)1;
+	    ep = (char *)NULL;
+	    if (!fp[22] || !*fp[22]
+	    ||  (rq = strtoul(fp[22], &ep, 0)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    rqs = (unsigned char)1;
+	    ep = (char *)NULL;
+	    if (!fp[4] || !*fp[4]
+	    ||  (state = strtoul(fp[4], &ep, 0)) == ULONG_MAX || !ep || *ep)
 		continue;
 	/*
 	 * Allocate space for the destination address.
 	 */
-	    if (!fp[0] || !*fp[0])
+	    if (!fp[3] || !*fp[3])
 		da = (char *)NULL;
-	    else if ((len = strlen(fp[0]))) {
+	    else if ((len = strlen(fp[3]))) {
 		if (!(da = (char *)malloc(len + 1))) {
 		    (void) fprintf(stderr,
 		      "%s: can't allocate %d destination AX25 addr bytes: %s\n",
-		      Pn, len + 1, fp[0]);
+		      Pn, len + 1, fp[3]);
 		    Exit(1);
 		}
-		(void) snpf(da, len + 1, "%s", fp[0]);
-	    }
+		(void) snpf(da, len + 1, "%s", fp[3]);
+	    } else
+		da = (char *)NULL;
 	/*
 	 * Allocate space for the source address.
 	 */
-	    if (!fp[1] || !*fp[1])
+	    if (!fp[2] || !*fp[2])
 		sa = (char *)NULL;
-	    else if ((len = strlen(fp[1]))) {
+	    else if ((len = strlen(fp[2]))) {
 		if (!(sa = (char *)malloc(len + 1))) {
 		    (void) fprintf(stderr,
 			"%s: can't allocate %d source AX25 address bytes: %s\n",
-			Pn, len + 1, fp[1]);
+			Pn, len + 1, fp[2]);
 		    Exit(1);
 		}
-		(void) snpf(sa, len + 1, "%s", fp[1]);
-	    }
+		(void) snpf(sa, len + 1, "%s", fp[2]);
+	    } else
+		sa = (char *)NULL;
 	/*
 	 * Allocate space for the device characters.
 	 */
-	    if (!fp[2] || !*fp[2])
+	    if (!fp[1] || !*fp[1])
 		dev_ch = (char *)NULL;
-	    else if ((len = strlen(fp[2]))) {
+	    else if ((len = strlen(fp[1]))) {
 		if (!(dev_ch = (char *)malloc(len + 1))) {
 		    (void) fprintf(stderr,
 		      "%s: can't allocate %d destination AX25 dev bytes: %s\n",
-		      Pn, len + 1, fp[2]);
+		      Pn, len + 1, fp[1]);
 		    Exit(1);
 		}
-		(void) snpf(dev_ch, len + 1, "%s", fp[2]);
-	    }
+		(void) snpf(dev_ch, len + 1, "%s", fp[1]);
+	    } else
+		dev_ch = (char *)NULL;
 	/*
 	 * Allocate space for an ax25sin entry, fill it, and link it to its
 	 * hash bucket.
@@ -669,6 +662,8 @@ get_net(p, pl)
 	(void) get_tcpudp(path, 1, 0);
 
 #if	defined(HASIPv6)
+	(void) make_proc_path(p, pl, &path, &pathl, "raw6");
+	(void) get_raw6(path);
 	(void) make_proc_path(p, pl, &path, &pathl, "tcp6");
 	(void) get_tcpudp6(path, 0, 1);
 	(void) make_proc_path(p, pl, &path, &pathl, "udp6");
@@ -967,6 +962,154 @@ get_tcpudp(p, pr, clr)
 
 
 #if	defined(HASIPv6)
+/*
+ * get_raw6() - get /proc/net/raw6 info
+ */
+
+static void
+get_raw6(p)
+	char *p;			/* /proc/net/raw path */
+{
+	char buf[MAXPATHLEN], *ep, **fp, *la, *ra, *sp;
+	int h;
+	unsigned long inode;
+	int nf = 12;
+	struct rawsin *np, *rp;
+	MALLOC_S lal, ral, spl;
+	FILE *xs;
+/*
+ * Do second time cleanup or first time setup.
+ */
+	if (Rawsin6) {
+	    for (h = 0; h < INOBUCKS; h++) {
+		for (rp = Rawsin6[h]; rp; rp = np) {
+		    np = rp->next;
+		    if (rp->la)
+			(void) free((FREE_P *)rp->la);
+		    if (rp->ra)
+			(void) free((FREE_P *)rp->ra);
+		    (void) free((FREE_P *)rp);
+		}
+		Rawsin6[h] = (struct rawsin *)NULL;
+	    }
+	} else {
+	    Rawsin6 = (struct rawsin **)calloc(INOBUCKS,
+					       sizeof(struct rawsin *));
+	    if (!Rawsin6) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d raw6 hash pointer bytes\n",
+		    Pn, INOBUCKS * sizeof(struct rawsin *));
+		Exit(1);
+	    }
+	}
+/*
+ * Open and read the /proc/net/raw6 file.  Store raw6 socket info in the
+ * Rawsin6[] hash buckets.
+ */
+	if (!(xs = fopen(p, "r")))
+	    return;
+	while (fgets(buf, sizeof(buf) - 1, xs)) {
+	    if (get_fields(buf, (char *)NULL, &fp) < nf)
+		continue;
+	    if (nf == 12) {
+
+	    /*
+	     * Check the column labels in the first line.
+	     */
+		if (!fp[1]  || strcmp(fp[1],  "local_address")
+		||  !fp[2]  || strcmp(fp[2],  "remote_address")
+		||  !fp[3]  || strcmp(fp[3],  "st")
+		||  !fp[11] || strcmp(fp[11], "inode"))
+		{
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: WARNING: unsupported format: %s\n",
+			    Pn, p);
+		    }
+		    break;
+		}
+		nf = 10;
+		continue;
+	    }
+	/*
+	 * Assemble the inode number and see if the inode is already
+	 * recorded.
+	 */
+	    ep = (char *)NULL;
+	    if (!fp[9] || !*fp[9]
+	    ||  (inode = strtoul(fp[9], &ep, 0)) == ULONG_MAX
+	    ||  !ep || *ep)
+		continue;
+	    h = INOHASH(inode);
+	    for (rp = Rawsin6[h]; rp; rp = rp->next) {
+		if (inode == rp->inode)
+		    break;
+	    }
+	    if (rp)
+		continue;
+	/*
+	 * Save the local address, remote address, and state.
+	 */
+	    if (!fp[1] || !*fp[1] || (lal = strlen(fp[1])) < 1) {
+		la = (char *)NULL;
+		lal = (MALLOC_S)0;
+	    } else {
+		if (!(la = (char *)malloc(lal + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d local raw6 address bytes: %s\n",
+			Pn, lal + 1, fp[1]);
+		    Exit(1);
+		}
+		(void) snpf(la, lal + 1, "%s", fp[1]);
+	    }
+	    if (!fp[2] || !*fp[2] || (ral = strlen(fp[2])) < 1) {
+		ra = (char *)NULL;
+		ral = (MALLOC_S)0;
+	    } else {
+		if (!(ra = (char *)malloc(ral + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d remote raw6 address bytes: %s\n",
+			Pn, ral + 1, fp[2]);
+		    Exit(1);
+		}
+		(void) snpf(ra, ral + 1, "%s", fp[2]);
+	    }
+	    if (!fp[3] || !*fp[3] || (spl = strlen(fp[3])) < 1) {
+		sp = (char *)NULL;
+		spl = (MALLOC_S)0;
+	    } else {
+		if (!(sp = (char *)malloc(spl + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d remote raw6 state bytes: %s\n",
+			Pn, spl + 1, fp[2]);
+		    Exit(1);
+		}
+		(void) snpf(sp, spl + 1, "%s", fp[3]);
+	    }
+	/*
+	 * Allocate space for an rawsin entry, fill it, and link it to its
+	 * hash bucket.
+	 */
+	    if (!(rp = (struct rawsin *)malloc(sizeof(struct rawsin)))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d byte rawsin structure for IPv6\n",
+		    Pn, sizeof(struct rawsin));
+		Exit(1);
+	    }
+	    rp->inode = inode;
+	    rp->la = la;
+	    rp->lal = lal;
+	    rp->ra = ra;
+	    rp->ral = ral;
+	    rp->sp = sp;
+	    rp->spl = spl;
+	    rp->next = Rawsin6[h];
+	    Rawsin6[h] = rp;
+	}
+	(void) fclose(xs);
+}
+
+
 /*
  * get_tcpudp6() - get IPv6 TCP or UDP IPv4 net info
  */
@@ -1224,7 +1367,26 @@ get_unix(p)
 	    }
 	    up->inode = inode;
 	    up->pcb = pcb;
-	    up->path = path;
+	    up->sb_def = 0;
+	    if ((up->path = path) && (*path == '/')) {
+
+	    /*
+	     * If an absolute path (i.e., one that begins with a '/') exists
+	     * for the line, attempt to stat(2) it and save the device and
+	     * node numbers reported in the stat buffer.
+	     */
+		struct stat sb;
+
+		if (((HasNFS && !statsafely(path, &sb))
+		||   (!HasNFS && !stat(path, &sb)))
+		&&  ((sb.st_mode & S_IFMT) == S_IFSOCK)
+		) {
+		    up->sb_def = 1;
+		    up->sb_dev = sb.st_dev;
+		    up->sb_ino = (unsigned long)sb.st_ino;
+		    up->sb_rdev = sb.st_rdev;
+		}
+	    }
 	    up->next = Uxsin[h];
 	    Uxsin[h] = up;
 	}
@@ -1257,7 +1419,7 @@ net6a2in6(as, ad)
 	    buf[8] = '\0';
 	    ep = (char *)NULL;
 	    if ((ad->s6_addr32[i] = (uint32_t)strtoul(buf, &ep, 16))
-	    ==  ULONG_MAX || !ep || *ep)
+	    ==  (uint32_t)UINT32_MAX || !ep || *ep)
 		break;
 	}
 	return((*as || (i != 4) || len) ? 1 : 0);
@@ -1267,31 +1429,45 @@ net6a2in6(as, ad)
 
 /*
  * print_ax25info() - print AX25 socket info
- *
- * DEBUG: this code hasn't been tested!
  */
 
 static void
 print_ax25info(ap)
 	struct ax25sin *ap;		/* AX25 socket info */
 {
-	char *cp, pbuf[256];
-	MALLOC_S pl;
+	char *cp, pbuf[1024];
+	int ds;
+	MALLOC_S pl = (MALLOC_S)0;
 
 	if (Lf->nma)
 	    return;
+	if (ap->sa) {
+	    ds = (ap->da && strcmp(ap->da, "*")) ? 1 : 0;
+	    (void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "%s%s%s ", ap->sa,
+		ds ? "->" : "",
+		ds ? ap->da : "");
+	    pl = strlen(pbuf);
+	}
 	if (ap->sqs) {
-	    (void) snpf(pbuf, sizeof(pbuf), "(Sq=%lu ", ap->sq);
+	    (void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "(Sq=%lu ", ap->sq);
 	    pl = strlen(pbuf);
-	} else {
-	    pbuf[0] = '(';
-	    pl = 1;
-	}
+	    cp = "";
+	} else 
+	    cp = "(";
 	if (ap->rqs) {
-	    (void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "Rq=%lu ", ap->rq);
+	    (void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "%sRq=%lu ", cp, ap->rq);
 	    pl = strlen(pbuf);
+	    cp = "";
 	}
-	(void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "State=%d)", ap->state);
+	(void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "%sState=%d", cp, ap->state);
+	pl = strlen(pbuf);
+	if ((ap->state >= 0) && (ap->state < NAX25ST))
+	    cp = ax25st[ap->state];
+	else
+	    cp = NULL;
+	(void) snpf(&pbuf[pl], sizeof(pbuf) - pl, "%s%s)",
+	    cp ? ", " : "",
+	    cp ? cp : "");
 	pl = strlen(pbuf);
 	if (!(cp = (char *)malloc(pl + 1))) {
 	    (void) fprintf(stderr,
@@ -1475,11 +1651,13 @@ print_tcptpi(nl)
  */
 
 void
-process_proc_sock(p, s, l)
+process_proc_sock(p, s, ss, l, lss)
 	char *p;			/* node's readlink() path */
 	struct stat *s;			/* stat() result for path */
+	int ss;				/* *s status -- i.e, SB_* values */
 	struct stat *l;			/* lstat() result for FD (NULL for
 					 * others) */
+	int lss;				/* *l status -- i.e, SB_* values */
 {
 	struct ax25sin *ap;
 	char *cp, dev_ch[32], *path;
@@ -1501,14 +1679,17 @@ process_proc_sock(p, s, l)
  * Enter offset, if possible.
  */
 	if (Foffset || !Fsize) {
-	    if (l)
+	    if (l && (lss & SB_SIZE)) {
 		Lf->off = (SZOFFTYPE)l->st_size;
-	    Lf->off_def = 1;
+		Lf->off_def = 1;
+	    }
 	}
 /*
  * Check for socket's inode presence in the protocol info caches.
  */
-	if ((tp = check_tcpudp((unsigned long)s->st_ino, &pr))) {
+	if ((ss & SB_INO)
+	&&  (tp = check_tcpudp((unsigned long)s->st_ino, &pr))
+	) {
 
 	/*
 	 * The inode is connected to an IPv4 TCP or UDP /proc record.
@@ -1518,7 +1699,7 @@ process_proc_sock(p, s, l)
 	 * save the local and foreign IPv4 addresses; save the type and
 	 * protocol; and (optionally) save the queue sizes.
 	 */
-	    if (Fnet)
+	    if (Fnet && (FnetTy != 6))
 		Lf->sf |= SELNET;
 
 #if	defined(HASIPv6)
@@ -1529,8 +1710,11 @@ process_proc_sock(p, s, l)
 
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
-	    (void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
-	    enter_dev_ch(dev_ch);
+	    if (ss & SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
+		    (unsigned long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
 	    if (tp->faddr || tp->fport) {
 		fs.s_addr = tp->faddr;
 		fa = (unsigned char *)&fs;
@@ -1555,7 +1739,9 @@ process_proc_sock(p, s, l)
 	}
 
 #if	defined(HASIPv6)
-	if ((tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))) {
+	if ((ss & SB_INO)
+	&&  (tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))
+	) {
 
 	/*
 	 * The inode is connected to an IPv6 TCP or UDP /proc record.
@@ -1565,13 +1751,16 @@ process_proc_sock(p, s, l)
 	 * and foreign IPv6 addresses; save the type and protocol; and
 	 * (optionally) save the queue sizes.
 	 */
-	    if (Fnet)
+	    if (Fnet && (FnetTy != 4))
 		Lf->sf |= SELNET;
 	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv6");
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
-	    (void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
-	    enter_dev_ch(dev_ch);
+	    if (ss && SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
+		    (unsigned long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
 	    af = AF_INET6;
 	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->faddr) || tp6->fport)
 		fa = (unsigned char *)&tp6->faddr;
@@ -1603,7 +1792,9 @@ process_proc_sock(p, s, l)
 	}
 #endif	/* defined(HASIPv6) */
 
-	if ((up = check_unix((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (up = check_unix((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to a UNIX /proc record.
@@ -1616,18 +1807,78 @@ process_proc_sock(p, s, l)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	    if (up->pcb)
 		enter_dev_ch(up->pcb);
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
 	    path = up->path ? up->path : p;
 	    (void) enter_nm(path);
-	    if (Sfile && is_file_named(path,
-				((s->st_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
-	    {
-		Lf->sf |= SELNM;
+	    if (Sfile) {
+	    
+	    /*
+	     * See if this UNIX domain socket was specified as a search
+	     * argument.
+	     *
+	     * Search first by device and node numbers, if that is possible;
+	     * then search by name.
+	     */
+		unsigned char f = 0;		/* file-found flag */
+
+		if (up->sb_def) {
+
+		/*
+		 * If the UNIX socket information includes stat(2) results, do
+		 * a device and node number search.
+		 *
+		 * Note: that requires the saving, temporary modification and
+		 *	 restoration of some *Lf values.
+		 */
+		    unsigned char sv_dev_def;	/* saved dev_def */
+		    unsigned char sv_inp_ty;	/* saved inp_ty */
+		    unsigned char sv_rdev_def;	/* saved rdev_def */
+		    dev_t sv_dev;		/* saved dev */
+		    unsigned long sv_inode;	/* saved inode */
+		    dev_t sv_rdev;		/* saved rdev */
+
+		    sv_dev_def = Lf->dev_def;
+		    sv_dev = Lf->dev;
+		    sv_inode = Lf->inode;
+		    sv_inp_ty = Lf->inp_ty;
+		    sv_rdev_def = Lf->rdev_def;
+		    sv_rdev = Lf->rdev;
+		    Lf->dev_def = Lf->inp_ty = Lf->rdev_def = 1;
+		    Lf->dev = up->sb_dev;
+		    Lf->inode = up->sb_ino;
+		    Lf->rdev = up->sb_rdev;
+		    if (is_file_named((char *)NULL, 0)) {
+			f = 1;
+			Lf->sf |= SELNM;
+		    }
+		    Lf->dev_def = sv_dev_def;
+		    Lf->dev = sv_dev;
+		    Lf->inode = sv_inode;
+		    Lf->inp_ty = sv_inp_ty;
+		    Lf->rdev_def = sv_rdev_def;
+		    Lf->rdev = sv_rdev;
+		}
+		if (!f && (ss & SB_MODE)) {
+
+		/*
+		 * If the file has not yet been found and the stat buffer has
+		 * st_mode, search for the file by full path.
+		 */
+		    if (is_file_named(path,
+			((s->st_mode & S_IFMT) == S_IFCHR)) ? 1 : 0)
+		    {
+			Lf->sf |= SELNM;
+		    }
+		}
 	    }
 	    return;
 	}
-	if ((ap = check_ax25((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (ap = check_ax25((unsigned long)s->st_ino))
+	) {
 	
 	/*
 	 * The inode is connected to an AX25 /proc record.
@@ -1635,8 +1886,6 @@ process_proc_sock(p, s, l)
 	 * Set the type to "ax25"; save the device name; save the inode number;
 	 * save the destination and source addresses; save the send and receive
 	 * queue sizes; and save the connection state.
-	 *
-	 * DEBUG: this code hasn't been tested!
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "ax25");
 	    if (ap->dev_ch)
@@ -1646,7 +1895,9 @@ process_proc_sock(p, s, l)
 	    print_ax25info(ap);
 	    return;
 	}
-	if ((ip = check_ipx((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (ip = check_ipx((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to an IPX /proc record.
@@ -1655,10 +1906,14 @@ process_proc_sock(p, s, l)
 	 * the addresses, queue sizes, and state in the NAME column.
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "ipx");
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
-	    Lf->dev = s->st_dev;
-	    Lf->dev_def = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
+	    if (ss & SB_DEV) {
+		Lf->dev = s->st_dev;
+		Lf->dev_def = 1;
+	    }
 	    cp = Namech;
 	    nl = MAXPATHLEN - 2;
 	    if (ip->la && nl) {
@@ -1695,7 +1950,9 @@ process_proc_sock(p, s, l)
 		enter_nm(Namech);
 	    return;
 	}
-	if ((rp = check_raw((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (rp = check_raw((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to a raw /proc record.
@@ -1704,8 +1961,10 @@ process_proc_sock(p, s, l)
 	 * address, remote address, and state in the NAME column.
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "raw");
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
 	    cp = Namech;
 	    nl = MAXPATHLEN - 2;
 	    if (rp->la && rp->lal) {
@@ -1754,9 +2013,13 @@ process_proc_sock(p, s, l)
  * The socket's protocol can't be identified.
  */
 	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
-	Lf->inode = (unsigned long)s->st_ino;
-	Lf->inp_ty = 1;
-	Lf->dev = s->st_dev;
-	Lf->dev_def = 1;
+	if (ss & SB_INO) {
+	    Lf->inode = (unsigned long)s->st_ino;
+	    Lf->inp_ty = 1;
+	}
+	if (ss & SB_DEV) {
+	    Lf->dev = s->st_dev;
+	    Lf->dev_def = 1;
+	}
 	enter_nm("can't identify protocol");
 }
