@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.22 2001/02/13 14:16:01 abe Exp $
+ * $Id: dlsof.h,v 1.35 2004/12/30 18:42:24 abe Exp $
  */
 
 
@@ -45,28 +45,76 @@
 #include <signal.h>
 #include <string.h>
 #include <unistd.h>
+
+# if	defined(HASGETBOOTFILE)
+#include <util.h>
+# endif	/* defined(HASGETBOOTFILE) */
+
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
+  
+# if	defined(HAS_LWP_H)
+#include <sys/lwp.h>
+# endif	/* defined(HAS_LWP_H) */
+
+# if	(defined(OPENBSDV) && OPENBSDV>=3030) \
+  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+#define	_KERNEL
+#  if	defined(NETBSDV)
+struct buf;	/* dummy for function prototype in <sys/buf.h> */
+struct uio;	/* dummy for function prototype in <sys/buf.h> */
+#  endif	/* defined(NETBSDV) */
+#include <sys/ucred.h>
+# endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
+	   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
 
 # if	defined(NETBSDV)
 #include <sys/buf.h>
 # endif	/* defined(NETBSDV) */
 
+# if	(defined(OPENBSDV) && OPENBSDV>=3030) \
+  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+#undef	_KERNEL
+# endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
+		   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
+
 #define	NFS
 #define m_stat	mnt_stat
+
+# if	(defined(OPENBSDV) && OPENBSDV>=3030) \
+  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+#define	_KERNEL
+# endif	/* (defined(OPENBSDV) && OPENBSDV<3030)
+	   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
+
 #include <sys/mount.h>
+
+# if	(defined(OPENBSDV) && OPENBSDV>=3030) \
+  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+#undef	_KERNEL
+# endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
+	   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
+
 #include <rpc/types.h>
 #include <sys/protosw.h>
 
-# if	defined(NETBSDV) && NETBSDV>=1030
+# if	defined(NETBSDV) && NETBSDV>=1003000
 #define	sockproto	NETBSD_sockproto
-# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+# endif	/* defined(NETBSDV) && NETBSDV>=1003000 */
 
 #include <sys/socket.h>
 
+# if	defined(HASMSDOSFS)
+#  if	HASMSDOSFS==1
 #include <msdosfs/bpb.h>
 #include <msdosfs/fat.h>
+#  else	/* HASMSDOSFS!=1 */
+#include <fs/msdosfs/bpb.h>
+#include <fs/msdosfs/fat.h>
+#  endif	/* HASMSDOSFS==1 */
 
+#  if	(defined(OPENBSDV) && OPENBSDV<3030) \
+   ||	(defined(NETBSDV) && __NetBSD_Version__<106060000)
 /*
  * As a terrible hack, the lsof Configure script extracts the netcred and
  * netexport structure definitions from <sys/mount.h> and places them in
@@ -80,19 +128,26 @@
  *
  * THIS IS A TERRIBLE AND FRAGILE HACK!!!  It might break if the netexport or
  * netcred definitions change radically in <sys/mount.h>.
+ *
+ * It is no longer needed for NetBSD Versions 1.6F and above, or for OpenBSD
+ * versions 3.3 and above.
  */
 
 #include "netexport.h"
+#  endif	/* (defined(OPENBSDV) && OPENBSDV<3030)
+		   || (defined(NETBSDV) && __NetBSD_Version__<106060000) */
+
 #define	_KERNEL
 struct nameidata;	/* to satisfy a function prototype in msdosfsmount.h */
 #include <msdosfs/msdosfsmount.h>
 #undef	_KERNEL
 #include <msdosfs/direntry.h>
 #include <msdosfs/denode.h>
+# endif	/* defined(HASMSDOSFS) */
 
-# if	defined(NETBSDV) && NETBSDV>=1030
+# if	defined(NETBSDV) && NETBSDV>=1003000
 #undef	sockproto
-# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+# endif	/* defined(NETBSDV) && NETBSDV>=1003000 */
 
 #include <sys/socketvar.h>
 #include <sys/un.h>
@@ -114,6 +169,26 @@ struct nameidata;	/* to satisfy a function prototype in msdosfsmount.h */
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
+
+# if	defined(OPENBSDV)
+#  if	!defined(TF_ECN_PERMIT)
+#define	TF_ECN_PERMIT	0x00008000	/* other side said I could ECN */
+#  endif	/* !defined(TF_ECN_PERMIT) */
+
+#  if	!defined(TF_RCVD_CE)
+#define	TF_RCVD_CE	0x00010000	/* send ECE in subsequent segs */ 
+#  endif	/* !defined(TF_RCVD_CE) */
+
+#  if	!defined(TF_SEND_CWR)
+#define	TF_SEND_CWR	0x00020000	/* send CWR in next seg */
+#  endif	/* !defined(TF_SEND_CWR) */
+
+#  if	!defined(TF_DISABLE_ECN)
+#define	TF_DISABLE_ECN	0x00040000	/* disable ECN for this connection */
+#  endif	/* !defined(TF_DISABLE_ECN) */
+
+# endif	/* defined(OPENBSDV) */
+
 #include <sys/ucred.h>
 
 # if	defined(UVM)
@@ -127,7 +202,7 @@ struct nameidata;	/* to satisfy a function prototype in msdosfsmount.h */
 
 #include <sys/vnode.h>
 
-# if	defined(NETBSDV) && NETBSDV>=1030
+# if	defined(NETBSDV) && NETBSDV>=1003000
 /*
  * Because late in the 1.3I NetBSD development cycle the sockproto structure
  * was placed under _KERNEL in <sys/socket.h>, and because defining _KERNEL
@@ -142,7 +217,7 @@ struct sockproto {
 	u_short sp_family;
 	u_short sp_protocol;
 };
-# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+# endif	/* defined(NETBSDV) && NETBSDV>=1003000 */
 
 #include <net/raw_cb.h>
 #include <sys/domain.h>
@@ -157,6 +232,12 @@ struct sockproto {
 #undef	DIRBLKSIZ
 # endif	/* defined(DIRBLKSIZ) */
 
+# if	defined(HASI_FFS1)
+#define	_KERNEL
+#include <ufs/ufs/ufsmount.h>
+#undef	_KERNEL
+# endif	/* defined(HASI_FFS1) */
+
 #include <ufs/ufs/inode.h>
 
 # if	defined(DIRENT_BLKSIZ)
@@ -164,6 +245,14 @@ struct sockproto {
 #undef	DIRENT_DIRBLKSIZ
 # endif	/*defined(DIRENT_BLKSIZ) */
 
+# if	defined(HASBUFQ_H)
+#  if	defined(NETBSDV) && NETBSDV>=2099010
+#define	_KERNEL
+#include <sys/bufq.h>
+#undef	_KERNEL
+#  endif	/* defined(NETBSDV) && NETBSDV>=2099010 */
+#endif	/* defined(HASBUFQ_H) */
+
 #undef KERNEL
 #include <ufs/mfs/mfsnode.h>
 
@@ -179,14 +268,31 @@ struct sockproto {
 #include <sys/proc.h>
 #include <kvm.h>
 #include <sys/sysctl.h>
+#  if	defined(HASKVMGETPROC2)
+#define	P_ADDR		p_paddr
+#define	P_COMM		p_comm
+#define	P_CWDI		p_cwdi
+#define	P_FD		p_fd
+#define	P_PID		p_pid
+#define	P_PGID		p__pgid
+#define	P_PPID		p_ppid
+#define	P_STAT		p_stat
+#define	P_TRACEP	p_tracep
+#define	P_UID		p_uid
+#define	P_VMSPACE	p_vmspace
+#  else	/* !defined(HASKVMGETPROC2) */
 #define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
+#define	P_CWDI		kp_proc.p_cwdi
 #define	P_FD		kp_proc.p_fd
 #define	P_PID		kp_proc.p_pid
 #define	P_PGID		kp_eproc.e_pgid
 #define	P_PPID		kp_eproc.e_ppid
 #define	P_STAT		kp_proc.p_stat
+#define	P_TRACEP	kp_proc.p_tracep
+#define	P_UID		kp_eproc.e_ucred.cr_uid
 #define	P_VMSPACE	kp_proc.p_vmspace
+#  endif	/* defined(HASKVMGETPROC2) */
 
 # if	defined(HASFDESCFS)
 #define	_KERNEL
@@ -196,15 +302,89 @@ struct sockproto {
 
 # if	defined(HASKERNFS)
 #define	_KERNEL
+#define	Pkern		__Pkern
+#define	Proot		__Proot
+#define	Pnull		__Pnull
+#define	Ptime		__Ptime
+#define	Pint		__Pint
+#define	Pstring		__Pstring
+#define	Phostname	__Phostname
+#define	Pavenrun	__Pavenrun
+#define	Pdevice		__Pdevice
+#define	Pmsgbuf		__Pmsgbuf
+#define	Pipsecsadir	__Pipsecsadir
+#define	Pipsecspdir	__Pipsecspdir
+#define	Pipsecsa	__Pipseca
+#define	Pipsecsp	__Pipsecsp
 #include <miscfs/kernfs/kernfs.h>
 #undef	_KERNEL
+#undef	Pkern
+#undef	Proot
+#undef	Pnull
+#undef	Ptime
+#undef	Pint
+#undef	Pstring
+#undef	Phostname
+#undef	Pavenrun
+#undef	Pdevice
+#undef	Pmsgbuf
+#undef	Pipsecsadir
+#undef	Pipsecspdir
+#undef	Pipseca
+#undef	Pipsecsp
+#  if	defined(HASKERNFS_KFS_KT)
+#define	kf_kt	kfs_kt
+#  endif	/* defined(HASKERNFS_KFS_KT) */
 # endif	/* defined(HASKERNFS) */
 
+# if	defined(HASNULLFS)
+#define	_KERNEL
+#  if	defined(NETBSDV) && NETBSDV>=1005000 && __NetBSD_Version__<106060000
+#include "netexport.h"
+#  endif	/* defined(NETBSDV) && NETBSDV>=1005000
+		   && __NetBSD_Version__<106060000 */
+#include <miscfs/nullfs/null.h>
+#undef	_KERNEL
+# endif	/* defined(HASNULLFS) */
+
 # if	defined(HASPROCFS)
+#  if	defined(HASPROCFS_PFSROOT)
+#define	_KERNEL
+#  endif	/* defined(HASPROCFS_PFSROOT) */
 #include <miscfs/procfs/procfs.h>
+#  if	defined(HASPROCFS_PFSROOT)
+#undef	_KERNEL
+#define	Proot		PFSroot
+#define	Pproc		PFSproc
+#define	Pcurproc	PFScurproc
+#define	Pmem		PFSmem
+#define	Pregs		PFSregs
+#define	Pfile		PFSfile
+#define	Pfpregs		PFSfpregs
+#define	Pctl		PFSctl
+#define	Pstatus		PFSstatus
+#define	Pnote		PFSnote
+#define	Pnotepg		PFSnotepg
+#   if	defined(NetBSDV)
+#    if	NETBSDV>=2000000
+#define	Pfd		PFSfd
+#    endif	/* NETBSDV>=2000000 */
+#    if	NETBSDV>=1006000
+#define	Pmap		PFSmap
+#define	Pmaps		PFSmaps
+#    endif	/* NETBSDV>=1006000 */
+#   endif	/* defined(NetBSDV) */
+#  endif	/* defined(HASPROCFS_PFSROOT) */
 #include <machine/reg.h>
 # endif	/* defined(HASPROCFS) */
 
+# if	defined(HASPTYFS)
+#define	_KERNEL
+#include <fs/ptyfs/ptyfs.h>
+#include <miscfs/specfs/specdev.h>
+#undef	_KERNEL
+# endif	/* defined(HASPTYFS) */
+
 #define	KERNEL
 #define _KERNEL
 #include <sys/file.h>
@@ -214,6 +394,13 @@ struct sockproto {
 struct vop_advlock_args;
 # endif	/* defined(HAS_ADVLOCK_ARGS) */
 
+# if	defined(DTYPE_KQUEUE)
+#define	HASKQUEUE				/* has the kqueue file type */
+#  if	defined(OPENBSDV)
+#include <sys/eventvar.h>
+#  endif	/* defined(OPENBSDV) */
+# endif	/* defined(DTYPE_KQUEUE) */
+
 #include <sys/lockf.h>
 #undef	KERNEL
 #undef	_KERNEL
@@ -240,7 +427,7 @@ struct vop_advlock_args;
 #include <vm/vm_pager.h>
 # endif	/* defined(HAS_UVM_INCL) */
 
-# if	defined(OPENBSDV)
+# if	defined(HAS_SYS_PIPEH)
 #  if	OPENBSDV==2030 && defined(__sparc__)
 #   if	defined(nbpg)
 #undef	nbpg
@@ -251,7 +438,7 @@ struct vop_advlock_args;
 				 * use of PIPE_NODIRECT in <sys/pipe.h>. */
 #  endif	/* OPENBSDV==2030 && defined(__sparc__) */
 #include <sys/pipe.h>
-#endif	/* defined(OPENBSDV) */
+#endif	/* defined(HAS_SYS_PIPEH) */
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
@@ -260,9 +447,13 @@ typedef	u_long		KA_T;
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
 #define MALLOC_S	size_t
+
+# if	defined(N_UNIXV)
 #define	N_UNIX_TMP(x)	#x
 #define	N_UNIX_STR(x)	N_UNIX_TMP(x)
 #define	N_UNIX		N_UNIX_STR(N_UNIXV)
+# endif	/* defined(N_UNIXV) */
+
 #define QSORT_P		void
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
@@ -310,7 +501,13 @@ struct mounts {
 #define	NL_NAME		n_name
 
 extern int Np;				/* number of kernel processes */
+
+# if	defined(HASKVMGETPROC2) 
+struct kinfo_proc2 *P;			/* local process table copy */
+# else	/* ! defined(HASKVMGETPROC2) */
 struct kinfo_proc *P;			/* local process table copy */
+# endif	/* defined(HASKVMGETPROC2) */
+
 extern int pgshift;			/* kernel's page shift */
 
 struct sfile {
@@ -338,13 +535,13 @@ struct sfile {
 
 
 /*
- * Definitions for rnam.c
+ * Definitions for rnam.c and rnmh.c
  */
 
 # if     defined(HASNCACHE)
-#  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020)
+#  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1002000)
 #include <stddef.h>
-#endif	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
+#endif	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1002000) */
 
 #include <sys/uio.h>
 #include <sys/namei.h>
@@ -354,15 +551,15 @@ struct sfile {
 #define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
 #define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
 
-#  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020)
+#  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1002000)
 #define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
-#  else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
+#  else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1002000) */
 #   if	defined(NetBSD1_0) && NetBSD<1994101
 #define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
 #   else	/* !defined(NetBSD1_0) || NetBSD>=1994101 */
 #define	NCACHE_NXT	nc_lru.tqe_next	/* link in NCACHE */
 #   endif	/* defined(NetBSD1_0) && NetBSD<1994101 */
-#  endif	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
+#  endif	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1002000) */
 
 #  if	defined(HASNCVPID)
 #define	NCACHE_PARID	nc_dvpid	/* parent node ID in NCACHE */
