@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.21 2001/02/13 14:16:14 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.34 2004/12/30 18:42:24 abe Exp $";
 #endif
 
 
@@ -60,16 +60,27 @@ getmemsz(pid)
 	pid_t pid;
 {
 	int n;
-	struct kinfo_proc *p;
 	struct vmspace vm;
 
+#if	defined(HASKVMGETPROC2)
+	struct kinfo_proc2 *p;
+#else	/* !defined(HASKVMGETPROC2) */
+	struct kinfo_proc *p;
+#endif	/* defined(HASKVMGETPROC2) */
+
 	for (n = 0, p = P; n < Np; n++, p++) {
 	    if (p->P_PID == pid) {
 		if (!p->P_VMSPACE
 		||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
 		    return;
+# if	defined(OPENBSDV)
+		Lf->sz = (SZOFFTYPE)((vm.vm_tsize + vm.vm_dsize
+		       + vm.vm_ssize) * sysconf(_SC_PAGESIZE));
+# else	/* !defined(OPENBSDV */
 		Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
 						     + vm.vm_ssize);
+# endif	/* defined(OPENBSDV) */
+
 		Lf->sz_def = 1;
 		return;
 	    }
@@ -125,6 +136,39 @@ lkup_dev_tty_again:
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
+#if	defined(HASKQUEUE)
+/*
+ * process_kqueue() -- process kqueue file
+ *
+ * Strictly speaking this function should appear in dfile.c, because it is
+ * a file processing function.  However, the Net and Open BSD sources don't
+ * require a dfile.c, so this is the next best location for the function.
+ */
+
+void
+process_kqueue(ka)
+	KA_T ka;			/* kqueue file structure address */
+{
+
+# if	defined(OPENBSDV)
+	struct kqueue kq;		/* kqueue structure */
+# endif	/* defined(OPENBSDV) */
+
+	(void) snpf(Lf->type, sizeof(Lf->type), "KQUEUE");
+	enter_dev_ch(print_kptr(ka, (char *)NULL, 0));
+
+# if	defined(OPENBSDV)
+	if (!ka || kread(ka, (char *)&kq, sizeof(kq)))
+	    return;
+	(void) snpf(Namech, Namechl, "count=%d, state=%#x", kq.kq_count,
+	    kq.kq_state);
+	enter_nm(Namech);
+# endif	/* defined(OPENBSDV) */
+
+}
+#endif	/* defined(HASKQUEUE) */
+
+
 /*
  * process_node() - process vnode
  */
@@ -134,21 +178,17 @@ process_node(va)
 	KA_T va;			/* vnode kernel space address */
 {
 	dev_t dev, rdev;
-	struct denode d;
-	u_long dpb;
-	unsigned char devs = 0;
+	unsigned char devs;
 	unsigned char lt;
 	unsigned char ns;
-	unsigned char rdevs = 0;
+	unsigned char rdevs;
 	char *ep, *ty;
 	struct lockf lf, *lff, *lfp;
 	struct inode i;
 	struct mfsnode m;
 	struct nfsnode n;
-	u_long long nn;
 	enum nodetype {NONODE, CDFSNODE, DOSNODE, EXT2NODE, FDESCNODE, INODE,
-		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE} nty = NONODE;
-	struct msdosfsmount pm;
+		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE, PTYFSNODE} nty;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
@@ -157,7 +197,7 @@ process_node(va)
 	dev_t iso_dev;
 	unsigned long iso_ino, iso_sz;
 	long iso_nlink;
-	int iso_stat = 0;
+	int iso_stat;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
@@ -171,15 +211,38 @@ process_node(va)
 
 #endif	/* defined(HASFDESCFS) */
 
+#if	defined(HASEXT2FS)
+# if	defined(HASI_E2FS_PTR)
+	struct ext2fs_dinode ed;
+# endif	/* defined(HASI_E2FS_PTR) */
+	struct ext2fs_dinode *edp = (struct ext2fs_dinode *)NULL;
+#endif	/* defined(HASEXT2FS) */
+
+#if	defined(HASI_FFS1)
+	unsigned char ffs = 0;
+	unsigned char u1s = 0;
+	unsigned char u2s = 0;
+	struct ufs1_dinode u1;
+	struct ufs2_dinode u2;
+	struct ufsmount um;
+#endif	/* defined(HASI_FFS1) */
+
 #if	defined(HASKERNFS)
 	struct kernfs_node kn;
 	struct stat ksb;
-	int ksbs = 0;
+	int ksbs;
 	struct kern_target kt;
 	int ktnl;
 	char ktnm[MAXPATHLEN+1];
 #endif	/* defined(HASKERNFS) */
 
+#if	defined(HASMSDOSFS)
+	struct denode d;
+	u_long dpb;
+	u_long nn;
+	struct msdosfsmount pm;
+#endif	/* defined(HASMSDOSFS) */
+
 #if	defined(HASNFSVATTRP)
 	struct vattr nv;
 #define	NVATTR	nv
@@ -187,12 +250,58 @@ process_node(va)
 #define	NVATTR	n.n_vattr
 #endif	/* defined(HASNFSVATTRP) */
 
+#if	defined(HASNULLFS)
+	struct null_node nu;
+	int sc = 0;
+	struct l_vfs *nvfs = (struct l_vfs *)NULL;
+#endif	/* defined(HASNULLFS) */
+
 #if	defined(HASPROCFS)
 	struct pfsnode p;
 	struct procfsid *pfi;
 	size_t sz;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	struct ptyfsnode pt;
+	struct specinfo si;
+#endif	/* defined(HASPTYFS) */
+
+#if	defined(HASNULLFS)
+
+process_overlaid_node:
+
+	if (++sc > 1024) {
+	    (void) snpf(Namech, Namechl, "too many overlaid nodes");
+	    enter_nm(Namech);
+	    return;
+	}
+#endif	/* defined(HASNULLFS) */
+
+/*
+ * Initialize miscellaneous variables.  This is done so that processing an
+ * overlaid node will be a fresh start.
+ */
+	devs = rdevs = 0;
+	nty = NONODE;
+	Namech[0] = '\0';
+
+#if	defined(HAS9660FS)
+	iso_stat = 0;
+#endif	/* defined(HAS9660FS) */
+
+#if	defined(HASKERNFS)
+	ksbs = 0;
+#endif	/* defined(HASKERNFS) */
+
+#if	defined(HASEXT2FS)
+	edp = (struct ext2fs_dinode *)NULL;
+#endif	/* defined(HASEXT2FS) */
+
+#if	defined(HASI_FFS1)
+	ffs = u1s = u2s = 0;
+#endif	/* defined(HASI_FFS1) */
+
 /*
  * Read the vnode.
  */
@@ -208,9 +317,9 @@ process_node(va)
 
 #if	defined(HASNCACHE)
 	Lf->na = va;
-# if	defined(HASNCAPID)
+# if	defined(HASNCVPID)
 	Lf->id = v->v_id;
-# endif	/* defined(HASNCAPID) */
+# endif	/* defined(HASNCVPID) */
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASFSTRUCT)
@@ -273,6 +382,19 @@ process_node(va)
 	    break;
 #endif	/* defined(HAS9660FS) */
 
+#if	defined(HASFDESCFS)
+	case VT_FDESC:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
+		(void) snpf(Namech, Namechl, "can't read fdescnode at: %x",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = FDESCNODE;
+	    break;
+#endif	/* defined(HASFDESCFS) */
+
 #if	defined(HASKERNFS)
 	case VT_KERNFS:
 	
@@ -337,6 +459,8 @@ process_node(va)
 	    }
 	    nty = MFSNODE;
 	    break;
+
+#if	defined(HASMSDOSFS)
 	case VT_MSDOSFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&d, sizeof(d))) {
@@ -347,6 +471,8 @@ process_node(va)
 	    }
 	    nty = DOSNODE;
 	    break;
+#endif	/* defined(HASMSDOSFS) */
+
 	case VT_NFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
@@ -369,18 +495,25 @@ process_node(va)
 	    nty = NFSNODE;
 	    break;
 
-#if	defined(HASFDESCFS)
-	case VT_FDESC:
+#if	defined(HASNULLFS)
+	case VT_NULL:
+	    if ((sc == 1) && vfs)
+		nvfs = vfs;
 	    if (!v->v_data
-	    ||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
-		(void) snpf(Namech, Namechl, "can't read fdescnode at: %x",
+	    ||  kread((KA_T)v->v_data, (char *)&nu, sizeof(nu))) {
+		(void) snpf(Namech, Namechl, "can't read null_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
-	    nty = FDESCNODE;
-	    break;
-#endif	/* defined(HASFDESCFS) */
+	    if (!nu.null_lowervp) {
+		(void) snpf(Namech, Namechl, "null_node overlays nothing");
+		enter_nm(Namech);
+		return;
+	    }
+	    va = (KA_T)nu.null_lowervp;
+	    goto process_overlaid_node;
+#endif	/* defined(HASNULLFS) */
 
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
@@ -395,10 +528,27 @@ process_node(va)
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if    defined(HASPTYFS)
+	case VT_PTYFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&pt, sizeof(pt))) {
+		(void) snpf(Namech, Namechl, "can't read ptyfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = PTYFSNODE;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASEXT2FS)
 	case VT_EXT2FS:
 #endif	/* defined(HASEXT2FS) */
 
+#if	defined(HASLFS)
+	case VT_LFS:
+#endif	/* defined(HASLFS) */
+
 	case VT_UFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
@@ -409,12 +559,55 @@ process_node(va)
 	    }
 
 #if	defined(HASEXT2FS)
-	    if (v->v_tag == VT_EXT2FS)
+	    if (v->v_tag == VT_EXT2FS) {
 		nty = EXT2NODE;
-	    else
+
+# if	defined(HASI_E2FS_PTR)
+		if (i.i_din.e2fs_din
+		&&  !kread((KA_T)i.i_din.e2fs_din, (char *)&ed, sizeof(ed)))
+		    edp = &ed;
+# else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
+		edp = &i.i_din.e2fs_din;
+#  else	/* HASEXT2FS>=2 */
+		edp = &i.i_e2din;
+#  endif	/* HASEXT2FS>=2 */
+# endif	/* defined(HASI_E2FS_PTR) */
+
+	    } else
 #endif	/* defined(HASEXT2FS) */
 
+	    {
 		nty = INODE;
+
+#if	defined(HASI_FFS1)
+		/*
+		 * If there are multiple FFS's, read the relevant structures.
+		 */
+		    if (i.i_ump
+		    &&  !kread((KA_T)i.i_ump, (char *)&um, sizeof(um))) {
+			if (um.um_fstype == UFS1) {
+			    ffs = 1;
+			    if (i.i_din.ffs1_din
+			    &&  !kread((KA_T)i.i_din.ffs1_din, (char *)&u1,
+				       sizeof(u1)))
+			    {
+				u1s = 1;
+			    }
+			} else if (um.um_fstype == UFS2) {
+			    ffs = 2;
+			    if (i.i_din.ffs2_din
+			    &&  !kread((KA_T)i.i_din.ffs2_din, (char *)&u2,
+				       sizeof(u2)))
+			    {
+				u2s = 1;
+			    }
+			}
+		    }
+#endif	/* defined(HASI_FFS1) */
+
+	    }
+
 	    if ((lff = i.i_lockf)) {
 
 	    /*
@@ -433,6 +626,18 @@ process_node(va)
 		    case F_POSIX:
 			if ((KA_T)lf.lf_id == Kpa)
 			    lt = 1;
+
+#if	defined(HAS_LWP_H) && !defined(HAS_LF_LWP)
+			else {
+
+			    struct lwp lw;
+
+			    if (!kread((KA_T)lf.lf_id, (char *)&lw, sizeof(lw))
+			    &&  (KA_T)lw.l_proc == Kpa)
+				lt = 1;
+			}
+#endif	/* defined(HAS_LWP_H) && !defined(HAS_LF_LWP) */
+
 			break;
 		    }
 		    if (!lt)
@@ -466,10 +671,13 @@ process_node(va)
 	type = v->v_type;
 	switch (nty) {
 
+
+#if	defined(HASMSDOSFS)
 	case DOSNODE:
 	    dev = d.de_dev;
 	    devs = 1;
 	    break;
+#endif	/* defined(HASMSDOSFS) */
 
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
@@ -500,18 +708,25 @@ process_node(va)
 #if	defined(HASEXT2FS)
 	case EXT2NODE:
 
-# if	defined(HASI_FFS)
-	    dev = i.i_dev;
-	    devs = 1;
-# else	/* !defined(HASI_FFS) */
 	    dev = i.i_dev;
 	    devs = 1;
 	    if ((type == VCHR) || (type == VBLK)) {
-		rdev = i.i_rdev;
-		rdevs = 1;
-	    }
-# endif	/* defined(HASI_FFS) */
 
+# if	defined(HASI_E2FS_PTR)
+		if (edp) {
+		    rdev = edp->e2di_rdev;
+		    rdevs = 1;
+		}
+# else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
+		rdev = i.i_din.e2fs_din.e2di_rdev;
+#  else	/* HASEXT2FS>=2 */
+		rdev = i.i_e2din.e2di_rdev;
+#  endif	/* HASEXT2FS>=2 */
+		rdevs = 1;
+# endif	/* defined(HASI_E2FS_PTR) */
+
+	    }
 	    break;
 #endif	/* defined(HASEXT2FS) */
 
@@ -522,24 +737,44 @@ process_node(va)
 
 #if	defined(HASI_FFS)
 		rdev = i.i_ffs_rdev;
+		rdevs = 1;
 #else	/* !defined(HASI_FFS) */
+# if	defined(HASI_FFS1)
+		if (ffs == 1) {
+		    if (u1s) {
+			rdev = u1.di_rdev;
+			rdevs = 1;
+		    }
+		} else if (ffs == 2) {
+		    if (u2s) {
+			rdev = u2.di_rdev;
+			rdevs = 1;
+		    }
+		}
+# else	/* !defined(HASI_FFS1) */
 		rdev = i.i_rdev;
+		rdevs = 1;
+# endif	/* defined(HASI_FFS1) */
 #endif	/* defined(HASI_FFS) */
 
-		rdevs = 1;
 	    }
 	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (vfs) {
+
+# if	defined(HASSTATVFS)
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
 		dev = (dev_t)vfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 		devs = 1;
 	    }
 	    break;
 #endif	/* defined(HASKERNFS) */
 
-
 #if	defined(HAS9660FS)
 	case CDFSNODE:
 	    if (iso_stat) {
@@ -552,11 +787,28 @@ process_node(va)
 	case NFSNODE:
 	    dev = NVATTR.va_fsid;
 	    devs = 1;
+
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (v->v_un.vu_specinfo
+	    &&  !kread((KA_T)v->v_un.vu_specinfo, (char *)&si, sizeof(si))) {
+		rdev = si.si_rdev;
+		rdevs = 1;
+	    }
+	    if (vfs) {
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+		devs = 1;
+	    }
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 /*
  * Obtain the inode number.
  */
 	switch (nty) {
+
+#if	defined(HASMSDOSFS)
 	case DOSNODE:
 	    if (d.de_pmp && !kread((KA_T)d.de_pmp, (char *)&pm, sizeof(pm))) {
 		dpb = (u_long)(pm.pm_BytesPerSec / sizeof(struct direntry));
@@ -576,16 +828,15 @@ process_node(va)
 		Lf->inp_ty = 1;
 	    }
 	    break;
+#endif	/* defined(HASMSDOSFS) */
 
 #if	defined(HASEXT2FS)
 	case EXT2NODE:
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-	    if (type != VBLK) {
-		Lf->inode = (unsigned long)i.i_number;
-		Lf->inp_ty = 1;
-	    }
+	    Lf->inode = (unsigned long)i.i_number;
+	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASKERNFS)
@@ -618,6 +869,19 @@ process_node(va)
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (pt.ptyfs_type == PTYFSptc) {
+		if (pt.ptyfs_fileno > 0x3fffffff)
+		    Lf->inode = (unsigned long)(pt.ptyfs_fileno & 0x3fffffff);
+		else
+		    Lf->inode = (unsigned long)(pt.ptyfs_fileno - 1);
+	    } else
+		Lf->inode = (unsigned long)pt.ptyfs_fileno;
+	    Lf->inp_ty = 1;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 
 /*
@@ -693,38 +957,66 @@ process_node(va)
 
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
-		    if (nty == INODE) {
+		    switch (nty) {
+		    case INODE:
 
 #if	defined(HASI_FFS)
+			
 			Lf->sz = (SZOFFTYPE)i.i_ffs_size;
+			Lf->sz_def = 1;
+			break;
 #else	/* !defined(HASI_FFS) */
+# if	defined(HASI_FFS1)
+
+			if (ffs == 1) {
+			    if (u1s) {
+				Lf->sz = (SZOFFTYPE)u1.di_size;
+				Lf->sz_def = 1;
+			    }
+			} else if (ffs == 2) {
+			    if (u2s) {
+				Lf->sz = (SZOFFTYPE)u2.di_size;
+				Lf->sz_def = 1;
+			    }
+			}
+			break;
+# else	/* !defined(HASI_FFS1) */
 			Lf->sz = (SZOFFTYPE)i.i_size;
+			Lf->sz_def = 1;
+# endif	/* defined(HASI_FFS1) */
 #endif	/* defined(HASI_FFS) */
 
-			Lf->sz_def = 1;
-		    } else if (nty == DOSNODE) {
+			break;
+
+
+#if	defined(HASMSDOSFS)
+		    case DOSNODE:
 			Lf->sz = (SZOFFTYPE)d.de_FileSize;
 			Lf->sz_def = 1;
-		    } else if (nty == MFSNODE) {
+			break;
+#endif	/* defined(HASMSDOSFS) */
+
+		    case MFSNODE:
 			Lf->sz = (SZOFFTYPE)m.mfs_size;
 			Lf->sz_def = 1;
-		    }
+			break;
 
 #if	defined(HASEXT2FS)
-		    else if (nty == EXT2NODE) {
-
-# if	defined(HASI_E2FS)
+		    case EXT2NODE:
+# if	defined(HASI_E2FS_PTR)
+			if (edp) {
+			    Lf->sz = (SZOFFTYPE)edp->e2di_size;
+			    Lf->sz_def = 1;
+			}
+# else	/* !defined(HASI_E2FS_PTR) */
 			Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
-# else	/* !defined(HASI_E2FS) */
-			Lf->sz = (SZOFFTYPE)i.i_size;
-# endif	/* defined(HASI_E2FS) */
-
 			Lf->sz_def = 1;
-		    }
+# endif	/* defined(HASI_E2FS_PTR) */
+			break;
 #endif	/* defined(HASEXT2FS) */
 
-		}
-		else if ((type == VCHR || type == VBLK) && !Fsize)
+		    }
+		} else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
 		break;
 	    }
@@ -760,27 +1052,57 @@ process_node(va)
 		}
 		break;
 	    case N_REGLR:
-		if (nty == INODE) {
+		switch (nty) {
+		case INODE:
 
+#if	defined(HASEFFNLINK)
+		    Lf->nlink = (long)i.HASEFFNLINK;
+#else	/* !defined(HASEFFNLINK) */
 # if	defined(HASI_FFS)
 		    Lf->nlink = (long)i.i_ffs_nlink;
 # else	/* !defined(HASI_FFS) */
+#  if	defined(HASI_FFS1)
+		    if (ffs == 1) {
+			if (u1s)
+			    Lf->nlink = (long)u1.di_nlink;
+		    } else if (ffs == 2) {
+			if (u2s)
+			    Lf->nlink = (long)u2.di_nlink;
+		    }
+#  else	/* !defined(HASI_FFS1) */
+
 		    Lf->nlink = (long)i.i_nlink;
+#  endif	/* defined(HASI_FFS1) */
 # endif	/* defined(HASI_FFS) */
+#endif	/* defined(HASEFFNLINK) */
 
 		    Lf->nlink_def = 1;
-		} else if (nty == DOSNODE) {
+		    break;
+
+#if	defined(HASMSDOSFS)
+		case DOSNODE:
 		    Lf->nlink = (long)d.de_refcnt;
 		    Lf->nlink_def = 1;
-		}
+		    break;
+#endif	/* defined(HASMSDOSFS) */
 
 #if	defined(HASEXT2FS)
-		else if (nty == EXT2NODE) {
-		    Lf->nlink - (long)i.i_e2fs_nlink;
+		case EXT2NODE:
+# if	defined(HASI_E2FS_PTR)
+		    if (edp) {
+			Lf->nlink = (long)edp->e2di_nlink;
+			Lf->nlink_def = 1;
+		    }
+# else	/* !defined(HASI_E2FS_PTR) */
+		    Lf->nlink = (long)i.i_e2fs_nlink;
 		    Lf->nlink_def = 1;
-		}
-#endif	/* defined(HASEXT2FS) */
+# endif	/* defined(HASI_E2FS_PTR) */
 
+		    break;
+	
+#endif	/* defined(HASEXT2FS) */
+		
+		}
 		break;
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
@@ -791,6 +1113,23 @@ process_node(va)
  */
 	if (Ntype == N_NFS && Fnfs)
 	    Lf->sf |= SELNFS;
+
+#if	defined(HASNULLFS)
+/*
+ * If there is a saved nullfs vfs pointer, propagate its device number.
+ */
+	if (nvfs) {
+
+# if	defined(HASSTATVFS)
+	    dev = nvfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
+	    dev = nvfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
+	    devs = 1;
+	}
+#endif	/* defined(HASNULLFS) */
+
 /*
  * Save the file system names.
  */
@@ -799,7 +1138,7 @@ process_node(va)
 	    Lf->fsdev = vfs->fsname;
 	}
 /*
- * Save the deice numbers and their states.
+ * Save the device numbers and their states.
  *
  * Format the vnode type, and possibly the device name.
  */
@@ -858,7 +1197,8 @@ process_node(va)
  * 	ioctl(fd, TIOCNOTTY) files;
  *	/kern files
  *	memory node files;
- *	/proc files.
+ *	/proc files;
+ *	ptyfs files.
  */
 
 	if (type == VBAD)
@@ -928,14 +1268,62 @@ process_node(va)
 		(void) snpf(ep, sz, "/%d/notepg", p.pfs_pid);
 		ty = "PGID";
 		break;
+
+# if	defined(Pfd)
+	    case Pfd:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fd", p.pfs_pid);
+		ty = "PFD";
+		break;
+# endif	/* defined(Pfd) */
+
+# if	defined(Pmap)
+	    case Pmap:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/map", p.pfs_pid);
+		ty = "PMAP";
+		break;
+# endif	/* defined(Pmap) */
+
+# if	defined(Pmaps)
+	    case Pmaps:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/maps", p.pfs_pid);
+		ty = "PMPS";
+		break;
+# endif	/* defined(Pmaps) */
+
 	    }
 	    if (ty)
 		(void) snpf(Lf->type, sizeof(Lf->type), ty);
-	    if (Namech[0])
-		enter_nm(Namech);
 	}
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	else if (nty == PTYFSNODE) {
+	    (void) snpf(Namech, Namechl, "%s", Lf->fsdir);
+	    Lf->nlink = 1;
+	    Lf->nlink_def = 1;
+	    switch (pt.ptyfs_type) {
+	    case PTYFSpts:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu", (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSptc:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu (master)",
+		    (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSroot:
+		Lf->sz = 512;
+		Lf->sz_def = 1;
+		break;
+	    }
+	    if (ty)
+		(void) snpf(Lf->type, sizeof(Lf->type), ty);
+	}
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASBLKDEV)
 /*
  * If this is a VBLK file and it's missing an inode number, try to
@@ -989,7 +1377,9 @@ process_node(va)
 	    if (Sfile && is_file_named((char *)NULL,
 				       ((type == VCHR) || (type == VBLK)) ? 1
 									  : 0))
+	    {
 		Lf->sf |= SELNM;
+	    }
 	    if (ns)
 		Namech[0] = '\0';
 	}
@@ -1001,7 +1391,7 @@ process_node(va)
 }
 
 
-#if	defined(OPENBSDV)
+#if	defined(HAS_SYS_PIPEH)
 /*
  * process_pipe() - process a file structure whose type is DTYPE_PIPE
  */
@@ -1052,4 +1442,4 @@ process_pipe(pa)
 	if (Namech[0])
 	    enter_nm(Namech);
 }
-#endif	/* defined(OPENBSDV) */
+#endif	/* defined(HAS_SYS_PIPEH) */
