@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: prfp.c,v 1.6 99/06/22 09:56:03 abe Exp $";
+static char *rcsid = "$Id: prfp.c,v 1.10 2004/12/30 18:40:10 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -75,8 +75,8 @@ process_file(fp)
  * Read file structure.
  */
 	if (kread((KA_T)fp, (char *)&f, sizeof(f))) {
-	    (void) sprintf(Namech, "can't read file struct from %s",
-		print_kptr(fp, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read file struct from %s",
+		print_kptr(fp, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -147,12 +147,36 @@ process_file(fp)
 #endif	/* defined(DTYPE_VNODE) */
 
 		if (!Selinet)
+
+#if	defined(HASF_VNODE)
+		    process_node((KA_T)f.f_vnode);
+#else	/* !defined(HASF_VNODE) */
 		    process_node((KA_T)f.f_data);
+#endif	/* defined(HASF_VNODE) */
+
 		return;
 	    case DTYPE_SOCKET:
 		process_socket((KA_T)f.f_data);
 		return;
 
+#if	defined(HASKQUEUE)
+	    case DTYPE_KQUEUE:
+		process_kqueue((KA_T)f.f_data);
+		return;
+#endif	/* defined(HASKQUEUE) */
+
+#if	defined(HASPSXSEM)
+	    case DTYPE_PSXSEM:
+		process_psxsem((KA_T)f.f_data);
+		return;
+#endif	/* defined(HASPSXSEM) */
+
+#if	defined(HASPSXSHM)
+	    case DTYPE_PSXSHM:
+		process_psxshm((KA_T)f.f_data);
+		return;
+#endif	/* defined(HASPSXSHM) */
+
 #if	defined(HASPRIVFILETYPE)
 	    case PRIVFILETYPE:
 		HASPRIVFILETYPE((KA_T)f.f_data);
@@ -161,8 +185,9 @@ process_file(fp)
 
 	    default:
 		if (f.f_type || f.f_ops) {
-		    (void) sprintf(Namech, "%s file struct, ty=%#x, op=%#x",
-			print_kptr(fp, (char *)NULL), f.f_type, f.f_ops);
+		    (void) snpf(Namech, Namechl,
+			"%s file struct, ty=%#x, op=%#x",
+			print_kptr(fp, (char *)NULL, 0), f.f_type, f.f_ops);
 		    enter_nm(Namech);
 		    return;
 		}
