@@ -32,12 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.19 2001/01/02 12:45:48 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.23 2004/10/17 21:39:23 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(HASWIDECHAR) && defined(WIDECHARINCL)
+#include WIDECHARINCL
+#endif	/* defined(HASWIDECHAR) && defined(WIDECHARINCL) */
+
 
 /*
  * Local definitions
@@ -58,18 +62,6 @@ _PROTOTYPE(static int dostat,(char *path, char *buf, int len));
 _PROTOTYPE(static int doreadlink,(char *path, char *buf, int len));
 _PROTOTYPE(static int doinchild,(int (*fn)(), char *fp, char *rbuf, int rbln));
 
-# if	!defined(GET_MAX_FD)
-/*
- * This is not strictly a prototype, but GET_MAX_FD is the name of the
- * function that, in lieu of getdtablesize(), returns the maximum file
- * descriptor plus one (or file descriptor count).  GET_MAX_FD may be
- * defined in the dialect's machine.h.  If it is not, the following
- * selects getdtablesize().
- */
-
-#define	GET_MAX_FD	getdtablesize
-# endif	/* !defined(GET_MAX_FD) */
-
 #if	defined(HASINTSIGNAL)
 _PROTOTYPE(static int handleint,(int sig));
 #else	/* !defined(HASINTSIGNAL) */
@@ -87,6 +79,12 @@ static pid_t Cpid = 0;			/* child PID */
 static jmp_buf Jmp_buf;			/* jump buffer */
 static int Pipes[] =			/* pipes for child process */
 	{ -1, -1, -1, -1 };
+static int CtSigs[] = { 0, SIGINT, SIGKILL };
+					/* child termination signals (in order
+					 * of application) -- the first is a
+					 * dummy to allow pipe closure to
+					 * cause the child to exit */
+#define	NCTSIGS	(sizeof(CtSigs) / sizeof(int))
 
 
 #if	defined(HASNLIST)
@@ -129,31 +127,66 @@ build_Nl(d)
 
 
 /*
- * childx() - make child process exit
+ * childx() - make child process exit (if possible)
  */
 
 void
 childx()
 {
+	int at, sx;
+	pid_t wpid;
+
 	if (Cpid > 1) {
-	    if (setjmp(Jmp_buf)) {
-		(void) alarm(0);
-		(void) signal(SIGALRM, SIG_DFL);
-		if (!Fwarn)
-		    (void) fprintf(stderr,
-			"%s: WARNING -- child process %d may be hung.\n",
-			Pn, (int)Cpid);
-	    } else {
-		(void) kill(Cpid, SIGINT);
-		(void) kill(Cpid, SIGKILL);
+
+	/*
+	 * First close the pipes to and from the child.  That should cause the
+	 * child to exit.  Compute alarm time shares.
+	 */
+	    (void) closePipes();
+	    if ((at = TmLimit / NCTSIGS) < TMLIMMIN)
+		at = TMLIMMIN;
+	/*
+	 * Loop, waiting for the child to exit.  After the first pass, help
+	 * the child exit by sending it signals.
+	 */
+	    for (sx = 0; sx < NCTSIGS; sx++) {
+		if (setjmp(Jmp_buf)) {
+
+		/*
+		 * An alarm has rung.  Disable further alarms.
+		 *
+		 * If there are more signals to send, continue the signal loop.
+		 *
+		 * If the last signal has been sent, issue a warning (unless
+		 * warninge have been suppressed) and exit the signal loop.
+		 */
+		    (void) alarm(0);
+		    (void) signal(SIGALRM, SIG_DFL);
+		    if (sx < (NCTSIGS - 1))
+			continue;
+		    if (!Fwarn)
+			(void) fprintf(stderr,
+			    "%s: WARNING -- child process %d may be hung.\n",
+			    Pn, (int)Cpid);
+		    break;
+	        }
+	    /*
+	     * Send the next signal to the child process, after the first pass
+	     * through the loop.
+	     *
+	     * Wrap the wait() with an alarm.
+	     */
+		if (sx)
+		    (void) kill(Cpid, CtSigs[sx]);
 		(void) signal(SIGALRM, handleint);
-		(void) alarm(TmLimit);
-		(void) wait(NULL);
+		(void) alarm(at);
+		wpid = (pid_t) wait(NULL);
 		(void) alarm(0);
 		(void) signal(SIGALRM, SIG_DFL);
+		if (wpid == Cpid)
+		    break;
 	    }
 	    Cpid = 0;
-	    (void) closePipes();
 	}
 }
 
@@ -229,26 +262,11 @@ doinchild(fn, fp, rbuf, rbln)
 	    if (setjmp(Jmp_buf)) {
 
 	    /*
-	     * An alarm signal has been received.
+	     * Process an alarm that has rung.
 	     */
 		(void) alarm(0);
 		(void) signal(SIGALRM, SIG_DFL);
-		if (setjmp(Jmp_buf)) {
-		    if (!Fwarn && Cpid)
-			(void) fprintf(stderr,
-			    "%s: WARNING -- child process %d may be hung.\n",
-			    Pn, (int)Cpid);
-		} else if (Cpid) {
-		    (void) kill(Cpid, SIGINT);
-		    (void) kill(Cpid, SIGKILL);
-		    (void) signal(SIGALRM, handleint);
-		    (void) alarm(TmLimit);
-		    (void) wait(NULL);
-		    (void) alarm(0);
-		    (void) signal(SIGALRM, SIG_DFL);
-		}
-		Cpid = 0;
-		(void) closePipes();
+		(void) childx();
 		errno = ETIMEDOUT;
 		return(1);
 	    } else if (!Cpid) {
@@ -1123,10 +1141,11 @@ safepup(c, cl)
 int
 safestrlen(sp, flags)
 	char *sp;			/* string pointer */
-	int flags;			/* flags: bit 0: 0 = no NL
-					 *		 1 = add trailing NL
-					 *	      1: 0 = ' ' printable
-					 *		 1 = ' ' not printable
+	int flags;			/* flags:
+					 *   bit 0: 0 (0) = no NL
+					 *	    1 (1) = add trailing NL
+					 *	 1: 0 (0) = ' ' printable
+					 *	    1 (2) = ' ' not printable
 					 */
 {
 	char c;
@@ -1158,23 +1177,69 @@ safestrprt(sp, fs, flags)
 	char *sp;			/* string to print pointer pointer */
 	FILE *fs;			/* destination stream -- e.g., stderr
 					 * or stdout */
-	int flags;			/* flags: bit 0: 0 = no NL
-					 *		 1 = add trailing NL
-					 *	      1: 0 = ' ' printable
-					 *		 1 = ' ' not printable
+	int flags;			/* flags:
+					 *   bit 0: 0 (0) = no NL
+					 *	    1 (1) = add trailing NL
+					 *	 1: 0 (0) = ' ' printable
+					 *	    1 (2) = ' ' not printable
+					 *	 2: 0 (0) = print string as is
+					 *	    1 (4) = surround string
+					 *		    with '"'
+					 *	 4: 0 (0) = print ending '\n'
+					 *	    1 (8) = don't print ending
+					 *		    '\n'
 					 */
 {
 	char c;
+	int ip, lnc, lnt, sl;
+
+#if	defined(HASWIDECHAR)
+	wchar_t w;
+	int wcmx = MB_CUR_MAX;
+#else	/* !defined(HASWIDECHAR) */
+	static int wcmx = 1;
+#endif	/* defined(HASWIDECHAR) */
 
 	c = (flags & 2) ? ' ' : '\0';
+	if (flags & 4)
+	    putc('"', fs);
 	if (sp) {
-	    for (; *sp; sp++) {
+	    for (sl = strlen(sp); *sp; sl -= lnc, sp += lnc) {
+
+#if	defined(HASWIDECHAR)
+		if (wcmx > 1) {
+		    lnc = mblen(sp, sl);
+		    if (lnc > 1) {
+			if ((mbtowc(&w, sp, sl) == lnc) && iswprint(w)) {
+			    for (lnt = 0; lnt < lnc; lnt++) {
+				putc((int)*(sp + lnt), fs);
+			    }
+			} else {
+			    for (lnt = 0; lnt < lnc; lnt++) {
+			        fputs(safepup((unsigned int)*(sp + lnt),
+					      (int *)NULL), fs);
+			    }
+			}
+			continue;
+		    } else
+			lnc = 1;
+		} else
+		    lnc = 1;
+#else	/* !defined(HASWIDECHAR) */
+		lnc = 1;
+#endif	/* defined(HASWIDECHAR) */
+
 		if (isprint((unsigned char)*sp) && *sp != c)
 		    putc((int)(*sp & 0xff), fs);
-		else
+		else {
+		    if ((flags & 8) && (*sp == '\n') && !*(sp + 1))
+			break;
 		    fputs(safepup((unsigned int)*sp, (int *)NULL), fs);
+		}
 	    }
 	}
+	if (flags & 4)
+	    putc('"', fs);
 	if (flags & 1)
 	    putc('\n', fs);
 }
@@ -1192,15 +1257,24 @@ safestrprtn(sp, len, fs, flags)
 					 * print */
 	FILE *fs;			/* destination stream -- e.g., stderr
 					 * or stdout */
-	int flags;			/* flags: bit 0: 0 = no NL
-					 *		 1 = add trailing NL
-					 *	      1: 0 = ' ' printable
-					 *		 1 = ' ' not printable
+	int flags;			/* flags:
+					 *   bit 0: 0 (0) = no NL
+					 *	    1 (1) = add trailing NL
+					 *	 1: 0 (0) = ' ' printable
+					 *	    1 (2) = ' ' not printable
+					 *	 2: 0 (0) = print string as is
+					 *	    1 (4) = surround string
+					 *		    with '"'
+					 *	 4: 0 (0) = print ending '\n'
+					 *	    1 (8) = don't print ending
+					 *		    '\n'
 					 */
 {
 	char c, *up;
 	int cl, i;
 
+	if (flags & 4)
+	    putc('"', fs);
 	if (sp) {
 	    c = (flags & 2) ? ' ' : '\0';
 	    for (i = 0; i < len && *sp; sp++) {
@@ -1208,6 +1282,8 @@ safestrprtn(sp, len, fs, flags)
 		    putc((int)(*sp & 0xff), fs);
 		    i++;
 		} else {
+		    if ((flags & 8) && (*sp == '\n') && !*(sp + 1))
+			break;
 		    up = safepup((unsigned int)*sp, &cl);
 		    if ((i + cl) > len)
 			break;
@@ -1219,6 +1295,8 @@ safestrprtn(sp, len, fs, flags)
 	    i = 0;
 	for (; i < len; i++)
 	    putc(' ', fs);
+	if (flags & 4)
+	    putc('"', fs);
 	if (flags & 1)
 	    putc('\n', fs);
 }
