@@ -32,13 +32,20 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.28 2001/02/13 02:09:44 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.38 2005/01/05 15:38:21 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * Local function prototypes
+ */
+
+_PROTOTYPE(static int is_file_sel,(struct lproc *lp, struct lfile *lf));
+
+
 /*
  * add_nma() - add to NAME column addition
  */
@@ -116,11 +123,11 @@ alloc_fflbuf(bp, al, lr)
 
 void
 alloc_lfile(nm, num)
-	char *nm;
-	int num;
+	char *nm;			/* file descriptor name (may be NULL) */
+	int num;			/* file descriptor number -- -1 if
+					 * none */
 {
-	char *cp;
-	struct fd_lst *fp;
+	int fds;
 
 	if (Lf) {
 /*
@@ -153,6 +160,24 @@ alloc_lfile(nm, num)
 	Lf->lts.type = -1;
 	Lf->nlink = 0l;
 
+#if	defined(HASSOOPT)
+	Lf->lts.kai = Lf->lts.ltm = 0;
+	Lf->lts.opt = Lf->lts.qlen = Lf->lts.qlim = (unsigned int)0;
+	Lf->lts.rbsz = Lf->lts.sbsz = (unsigned long)0;
+	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		      = Lf->lts.sbszs = (unsigned char)0;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	Lf->lts.ss = 0;
+#endif	/* defined(HASSOSTATE) */
+
+#if	defined(HASTCPOPT)
+	Lf->lts.mss = (unsigned long)0;
+	Lf->lts.msss = (unsigned char)0;
+	Lf->lts.topt = (unsigned int)0;
+#endif	/* defined(HASTCPOPT) */
+
 #if	defined(HASTCPTPIQ)
 	Lf->lts.rqs = Lf->lts.sqs = (unsigned char)0;
 #endif	/* defined(HASTCPTPIQ) */
@@ -214,22 +239,15 @@ alloc_lfile(nm, num)
  */
 	if (!Fdl || (!nm && num < 0))
 	    return;
-	if ((cp = nm)) {
-	    while (*cp && *cp == ' ')
-		cp++;
-	}
-	for (fp = Fdl; fp; fp = fp->next) {
-	    if (cp) {
-		if (fp->nm && strcmp(fp->nm, cp) == 0) {
-		    Lf->sf |= SELFD;
-		    return;
-		}
-		continue;
-	    }
-	    if (num >= fp->lo && num <= fp->hi) {
+	fds = ck_fd_status(nm, num);
+	switch (FdlTy) {
+	case 0:			/* inclusion list */
+	    if (fds == 2)
+		Lf->sf |= SELFD;
+	    break;
+	case 1:			/* exclusion list */
+	    if (fds != 1)
 		Lf->sf |= SELFD;
-		return;
-	    }
 	}
 }
 
@@ -288,6 +306,68 @@ alloc_lproc(pid, pgid, ppid, uid, cmd, pss, sf)
 	    safestrprt(cmd, stderr, 1);
 	    Exit(1);
 	}
+
+#if	defined(HASZONES)
+/*
+ * Clear the zone name pointer.  The dialect's own code will set it.
+ */
+	Lp->zn = (char *)NULL;
+#endif	/* defined(HASZONES) */
+
+}
+
+
+/*
+ * ck_fd_status() - check FD status
+ *
+ * return: 0 == FD is neither included nor excluded
+ *	   1 == FD is excluded
+ *	   2 == FD is included
+ */
+
+extern int
+ck_fd_status(nm, num)
+	char *nm;			/* file descriptor name (may be NULL) */
+	int num;			/* file descriptor number -- -1 if
+					 * none */
+{
+	char *cp;
+	struct fd_lst *fp;
+
+	if (!(fp = Fdl) || (!nm && num < 0))
+	    return(0);
+	if ((cp = nm)) {
+	    while (*cp && *cp == ' ')
+		cp++;
+	}
+/*
+ * Check for an exclusion match.
+ */
+	if (FdlTy == 1) {
+	    for (; fp; fp = fp->next) {
+		if (cp) {
+		    if (fp->nm && strcmp(fp->nm, cp) == 0)
+			return(1);
+		    continue;
+		}
+		if (num >= fp->lo && num <= fp->hi)
+		    return(1);
+	    }
+	    return(0);
+	}
+/*
+ * If Fdl isn't an exclusion list, check for an inclusion match.
+ */
+	for (; fp; fp = fp->next) {
+	    if (cp) {
+		if (fp->nm && strcmp(fp->nm, cp) == 0)
+		    return(2);
+		continue;
+	    }
+	    if (num >= fp->lo && num <= fp->hi)
+		return(2);
+	}
+	return(0);
 }
 
 
@@ -358,7 +438,8 @@ ent_inaddr(la, lp, fa, fp, af)
  */
 	if ((Selflags & SELNA) && Nwad) {
 	    m = (fa && is_nw_addr(fa, fp, af)) ? 1 : 0;
-	    if ((m |= (la && is_nw_addr(la, lp, af)) ? 1 : 0))
+	    m |= (la && is_nw_addr(la, lp, af)) ? 1 : 0;
+	    if (m)
 		Lf->sf |= SELNA;
 	}
 }
@@ -505,16 +586,25 @@ is_cmd_excl(cmd, pss, sf)
  * is_file_sel() - is file selected?
  */
 
-int
-is_file_sel(lf)
+static int
+is_file_sel(lp, lf)
+	struct lproc *lp;		/* lproc structure pointer */
 	struct lfile *lf;		/* lfile structure pointer */
 {
-	if ( !lf || !lf->sf)
+	if (!lf || !lf->sf)
+	    return(0);
+
+#if	defined(HASSECURITY) && defined(HASNOSOCKSECURITY)
+	if (Myuid && (Myuid != lp->uid)) {
+	    if (!(lf->sf & (SELNA | SELNET)))
 		return(0);
+	}
+#endif	/* defined(HASSECURITY) && defined(HASNOSOCKSECURITY) */
+
 	if (Selall)
-		return(1);
+	    return(1);
 	if (Fand && ((lf->sf & Selflags) != Selflags))
-		return(0);
+	    return(0);
 	return(1);
 }
 
@@ -538,10 +628,14 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 #if	defined(HASSECURITY)
 /*
  * The process is excluded by virtue of the security option if it
- * isn't owned by the owner of this lsof process.
+ * isn't owned by the owner of this lsof process, unless the
+ * HASNOSOCKSECURITY option is also specified.  In that case the
+ * selected socket files of any process may be listed.
  */
+# if	!defined(HASNOSOCKSECURITY)
 	if (Myuid && Myuid != (uid_t)uid)
 	    return(1);
+# endif	/* !defined(HASNOSOCKSECURITY) */
 #endif	/* defined(HASSECURITY) */
 
 /*
@@ -558,12 +652,21 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 	    }
 	}
 /*
- * If the listing of all processes is selected, then this one
- * is not excluded.
+ * If the listing of all processes is selected, then this one is not excluded.
+ *
+ * However, if HASSECURITY and HASNOSOCKSECURITY are both specified, exclude
+ * network selections from the file flags, so that the tests in is_file_sel()
+ * work as expected.
  */
 	if (Selall) {
 	    *pss = PS_PRI;
+
+#if	defined(HASSECURITY) && defined(HASNOSOCKSECURITY)
+	    *sf = SELALL & ~(SELNA | SELNET);
+#else	/* !defined(HASSECURITY) || !defined(HASNOSOCKSECURITY) */
 	    *sf = SELALL;
+#endif	/* defined(HASSECURITY) && defined(HASNOSOCKSECURITY) */
+
 	    return(0);
 	}
 /*
@@ -651,7 +754,7 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 	}
 /*
  * Finally, when neither the process group ID, nor the PID, nor the UID is
- * selected, and no process group ID, PIDm or UID list option has been
+ * selected, and no process group ID, PID or UID list option has been
  * specified:
  *
  *	If list option ANDing has been specified, this process is
@@ -672,14 +775,14 @@ link_lfile()
 {
 	Lp->pss |= PS_SEC;
 	if (Plf)
-		Plf->next = Lf;
+	    Plf->next = Lf;
 	else
-		Lp->file = Lf;
+	    Lp->file = Lf;
 	Plf = Lf;
 	if (Fnet && (Lf->sf & SELNET))
-		Fnet = 2;
+	    Fnet = 2;
 	if (Fnfs && (Lf->sf & SELNFS))
-		Fnfs = 2;
+	    Fnfs = 2;
 	Lf = (struct lfile *)NULL;
 }
 
@@ -765,9 +868,10 @@ print_fflags(ffg, pof)
 int
 print_proc()
 {
-	char *cp;
-	int lc, st, ty;
+	char buf[128], *cp;
+	int lc, len, st, ty;
 	int rv = 0;
+	unsigned long ul;
 /*
  * If nothing in the process has been selected, skip it.
  */
@@ -776,22 +880,18 @@ print_proc()
 	if (Fterse) {
 
 	/*
-	 * The mode is terse and something in the process has been
-	 * selected.  If options are being OR'd, print the PID;
-	 * if AND'd, see if anything has been selected.
+	 * The mode is terse and something in the process appears to have
+	 * been selected.  Make sure of that by looking for a selected file,
+	 * so that the HASSECURITY and HASNOSOCKSECURITY option combination
+	 * won't produce a false positive result.
 	 */
-	    if (Fand) {
-		for (Lf = Lp->file; Lf; Lf = Lf->next) {
-		    if (is_file_sel(Lf)) {
-			rv = 1;
-			break;
-		    }
+	    for (Lf = Lp->file; Lf; Lf = Lf->next) {
+		if (is_file_sel(Lp, Lf)) {
+		    (void) printf("%d\n", Lp->pid);
+		    return(1);
 		}
-	    } else
-		rv = 1;
-	    if (rv)
-		(void) printf("%d\n", Lp->pid);
-	    return(rv);
+	    }
+	    return(0);
 	}
 /*
  * If fields have been selected, output the process-only ones, provided
@@ -799,13 +899,19 @@ print_proc()
  */
 	if (Ffield) {
 	    for (Lf = Lp->file; Lf; Lf = Lf->next) {
-		if (is_file_sel(Lf))
+		if (is_file_sel(Lp, Lf))
 		    break;
 	    }
 	    if (!Lf)
 		return(rv);
 	    rv = 1;
 	    (void) printf("%c%d%c", LSOF_FID_PID, Lp->pid, Terminator);
+
+#if	defined(HASZONES)
+	    if (FieldSel[LSOF_FIX_ZONE].st && Fzone && Lp->zn)
+		(void) printf("%c%s%c", LSOF_FID_ZONE, Lp->zn, Terminator);
+#endif	/* defined(HASZONES) */
+
 	    if (FieldSel[LSOF_FIX_PGID].st && Fpgid)
 		(void) printf("%c%d%c", LSOF_FID_PGID, Lp->pgid, Terminator);
 
@@ -833,7 +939,7 @@ print_proc()
  * Print files.
  */
 	for (Lf = Lp->file; Lf; Lf = Lf->next) {
-	    if (!is_file_sel(Lf))
+	    if (!is_file_sel(Lp, Lf))
 		continue;
 	    rv = 1;
 	/*
@@ -909,19 +1015,57 @@ print_proc()
 		}
 	    }
 	    if (FieldSel[LSOF_FIX_DEVN].st && Lf->dev_def) {
-		(void) printf("%c0x%lx%c", LSOF_FID_DEVN,
-		    (unsigned long)Lf->dev, Terminator);
+		if (sizeof(unsigned long) > sizeof(dev_t))
+		    ul = (unsigned long)((unsigned int)Lf->dev);
+		else
+		    ul = (unsigned long)Lf->dev;
+		(void) printf("%c0x%lx%c", LSOF_FID_DEVN, ul, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_RDEV].st && Lf->rdev_def) {
+		if (sizeof(unsigned long) > sizeof(dev_t))
+		    ul = (unsigned long)((unsigned int)Lf->rdev);
+		else
+		    ul = (unsigned long)Lf->rdev;
+		(void) printf("%c0x%lx%c", LSOF_FID_RDEV, ul, Terminator);
 		lc++;
 	    }
 	    if (FieldSel[LSOF_FIX_SIZE].st && Lf->sz_def) {
 		putchar(LSOF_FID_SIZE);
-		(void) printf(SzOffFmt_d, Lf->sz);
+
+#if	defined(HASPRINTSZ)
+		cp = HASPRINTSZ(Lf);
+#else	/* !defined(HASPRINTSZ) */
+		(void) snpf(buf, sizeof(buf), SzOffFmt_d, Lf->sz);
+		cp = buf;
+#endif	/* defined(HASPRINTSZ) */
+
+		(void) printf("%s", cp);
 		putchar(Terminator);
 		lc++;
 	    }
 	    if (FieldSel[LSOF_FIX_OFFSET].st && Lf->off_def) {
 		putchar(LSOF_FID_OFFSET);
-		(void) printf(SzOffFmt_d, Lf->off);
+
+#if	defined(HASPRINTOFF)
+		cp = HASPRINTOFF(Lf, 0);
+#else	/* !defined(HASPRINTOFF) */
+		(void) snpf(buf, sizeof(buf), SzOffFmt_0t, Lf->off);
+		cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		len = strlen(cp);
+		if (OffDecDig && len > (OffDecDig + 2)) {
+
+#if	defined(HASPRINTOFF)
+		    cp = HASPRINTOFF(Lf, 1);
+#else	/* !defined(HASPRINTOFF) */
+		    (void) snpf(buf, sizeof(buf), SzOffFmt_x, Lf->off);
+		    cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		}
+		(void) printf("%s", cp);
 		putchar(Terminator);
 		lc++;
 	    }
