@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: store.c,v 1.26 2001/02/13 02:10:43 abe Exp $";
+static char *rcsid = "$Id: store.c,v 1.33 2004/07/06 19:09:32 abe Exp $";
 #endif
 
 
@@ -59,9 +59,10 @@ struct clone *Clone = (struct clone *)NULL;
 				/* clone device list */
 #endif	/* defined(HAS_STD_CLONE) */
 
+int CmdColW;			/* COMMAND column width */
 struct str_lst *Cmdl = (struct str_lst *)NULL;
 				/* command names selected with -c */
-int CmdColW;			/* COMMAND column width */
+int CmdLim = CMDL;		/* COMMAND column width limit */
 lsof_rx_t *CmdRx = (lsof_rx_t *)NULL;
 				/* command regular expression table */
 
@@ -97,7 +98,7 @@ int DCunsafe = 0;		/* device cache file is potentially unsafe,
 int DChelp = 0;			/* -D? status */
 
 int DevColW;			/* DEVICE column width */
-dev_t DevDev;			/* devce number of /dev or its equivalent */
+dev_t DevDev;			/* device number of /dev or its equivalent */
 struct l_dev *Devtp = (struct l_dev *)NULL;
 				/* device table pointer */
 
@@ -111,6 +112,7 @@ int Dstkx = 0;			/* Dstk[] index */
 int Dstkn = 0;			/* Dstk[] entries allocated */
 
 int ErrStat = 0;		/* path stat() error count */
+uid_t Euid;			/* effective UID of this lsof process */
 int Fand = 0;			/* -a option status */
 int Fblock = 0;			/* -b option status */
 int FdColW;			/* FD column width */
@@ -127,8 +129,12 @@ int NcacheReload = 1;		/* 1 == call ncache_load() */
 int Ffield = 0;			/* -f and -F status */
 int Fhelp = 0;			/* -h option status */
 int Fhost = 1;			/* -H option status */
-int Fnet = 0;			/* -i option status: 0==none, 1==find all,
-				 * 2==some found*/
+int Fnet = 0;			/* -i option status: 0==none
+				 *		     1==find all
+				 *		     2==some found*/
+int FnetTy = 0;			/* Fnet type request: 0==all
+				 *		      4==IPv4
+				 *		      6==IPv6 */
 int Fnfs = 0;			/* -N option status: 0==none, 1==find all,
 				 * 2==some found*/
 int Fnlink = 0;			/* -L option status */
@@ -168,8 +174,14 @@ int Fwarn = 0;			/* +|-w option status */
 int Fxopt = HASXOPT_VALUE;	/* -X option status */
 #endif	/* defined(HASXOPT_VALUE) */
 
+int Fxover = 0;			/* -x option value */
+int Fzone = 0;			/* -z option status */
+
 struct fd_lst *Fdl = (struct fd_lst *)NULL;
 				/* file descriptors selected with -d */
+int FdlTy = -1;			/* Fdl[] type: -1 == none
+				 *		0 == include
+				 *		1 == exclude */
 
 struct fieldsel FieldSel[] = {
     { LSOF_FID_ACCESS, 0,  LSOF_FNM_ACCESS, NULL,     0		 }, /*  0 */
@@ -191,13 +203,15 @@ struct fieldsel FieldSel[] = {
     { LSOF_FID_PID,    1,  LSOF_FNM_PID,    NULL,     0		 }, /* 16 */
     { LSOF_FID_PGID,   0,  LSOF_FNM_PGID,   &Fpgid,   1		 }, /* 17 */
     { LSOF_FID_PROTO,  0,  LSOF_FNM_PROTO,  NULL,     0		 }, /* 18 */
-    { LSOF_FID_PPID,   0,  LSOF_FNM_PPID,   &Fppid,   1		 }, /* 19 */
-    { LSOF_FID_SIZE,   0,  LSOF_FNM_SIZE,   NULL,     0		 }, /* 20 */
-    { LSOF_FID_STREAM, 0,  LSOF_FNM_STREAM, NULL,     0		 }, /* 21 */
-    { LSOF_FID_TYPE,   0,  LSOF_FNM_TYPE,   NULL,     0		 }, /* 22 */
-    { LSOF_FID_TCPTPI, 0,  LSOF_FNM_TCPTPI, &Ftcptpi, TCPTPI_ALL }, /* 23 */
-    { LSOF_FID_UID,    0,  LSOF_FNM_UID,    NULL,     0		 }, /* 24 */
-    { LSOF_FID_TERM,   0,  LSOF_FNM_TERM,   NULL,     0		 }, /* 25 */
+    { LSOF_FID_RDEV,   0,  LSOF_FNM_RDEV,   NULL,     0		 }, /* 19 */
+    { LSOF_FID_PPID,   0,  LSOF_FNM_PPID,   &Fppid,   1		 }, /* 20 */
+    { LSOF_FID_SIZE,   0,  LSOF_FNM_SIZE,   NULL,     0		 }, /* 21 */
+    { LSOF_FID_STREAM, 0,  LSOF_FNM_STREAM, NULL,     0		 }, /* 22 */
+    { LSOF_FID_TYPE,   0,  LSOF_FNM_TYPE,   NULL,     0		 }, /* 23 */
+    { LSOF_FID_TCPTPI, 0,  LSOF_FNM_TCPTPI, &Ftcptpi, TCPTPI_ALL }, /* 24 */
+    { LSOF_FID_UID,    0,  LSOF_FNM_UID,    NULL,     0		 }, /* 25 */
+    { LSOF_FID_ZONE,   0,  LSOF_FNM_ZONE,   &Fzone,   1		 }, /* 26 */
+    { LSOF_FID_TERM,   0,  LSOF_FNM_TERM,   NULL,     0		 }, /* 27 */
 
 #if	defined(HASFIELDAP1)
     { '1',	       0,  HASFIELDAP1,     NULL,     0		 }, /* TERM+1 */
@@ -246,6 +260,10 @@ struct lproc *Lp = (struct lproc *)NULL;
 struct lproc *Lproc = (struct lproc *)NULL;
 				/* local process table */
 char *Memory = (char *)NULL;	/* core file path */
+int MntSup = 0;			/* mount supplement state: 0 == none
+				 *			   1 == create
+				 *			   2 == read */
+char *MntSupP = (char *)NULL;	/* mount supplement path -- if MntSup == 2 */
 
 #if	defined(HASPROCFS)
 struct mounts *Mtprocfs = (struct mounts *)NULL;
@@ -336,3 +354,10 @@ char Terminator = '\n';		/* output field terminator */
 int TmLimit = TMLIMIT;		/* Readlink() and stat() timeout (seconds) */
 int TypeColW;			/* TYPE column width */
 int UserColW;			/* USER column width */
+
+#if	defined(HASZONES)
+znhash_t **ZoneArg = (znhash_t **)NULL;
+				/* zone arguments supplied with -z */
+#endif	/* defined(HASZONES) */
+
+int ZoneColW;			/* ZONE column width */
